from collections import defaultdict
from typing import Any

import datasets
from datasets import DatasetDict
from tqdm.auto import tqdm

from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata

_FLEURS_EVAL_LANGS = {
    "af_za": ["afr-Latn"],  # Afrikaans
    "am_et": ["amh-Ethi"],  # Amharic
    "ar_eg": ["ara-Arab"],  # Arabic
    "as_in": ["asm-Beng"],  # Assamese
    "ast_es": ["ast-Latn"],  # Asturian
    "az_az": ["aze-Latn"],  # Azerbaijani
    "be_by": ["bel-Cyrl"],  # Belarusian
    "bn_in": ["ben-Beng"],  # Bengali
    "bs_ba": ["bos-Latn"],  # Bosnian
    "ca_es": ["cat-Latn"],  # Catalan
    "ceb_ph": ["ceb-Latn"],  # Cebuano
    "cmn_hans_cn": ["cmn-Hans"],  # Mandarin Chinese (Simplified)
    "yue_hant_hk": ["yue-Hant"],  # Cantonese Chinese (Traditional)
    "cs_cz": ["ces-Latn"],  # Czech
    "cy_gb": ["cym-Latn"],  # Welsh
    "da_dk": ["dan-Latn"],  # Danish
    "de_de": ["deu-Latn"],  # German
    "el_gr": ["ell-Grek"],  # Greek
    "en_us": ["eng-Latn"],  # English
    "es_419": ["spa-Latn"],  # Spanish (Latin America)
    "et_ee": ["est-Latn"],  # Estonian
    "fa_ir": ["fas-Arab"],  # Persian
    "ff_sn": ["ful-Latn"],  # Fula
    "fi_fi": ["fin-Latn"],  # Finnish
    "fil_ph": ["fil-Latn"],  # Filipino
    "fr_fr": ["fra-Latn"],  # French
    "ga_ie": ["gle-Latn"],  # Irish
    "gl_es": ["glg-Latn"],  # Galician
    "gu_in": ["guj-Gujr"],  # Gujarati
    "ha_ng": ["hau-Latn"],  # Hausa
    "he_il": ["heb-Hebr"],  # Hebrew
    "hi_in": ["hin-Deva"],  # Hindi
    "hr_hr": ["hrv-Latn"],  # Croatian
    "hu_hu": ["hun-Latn"],  # Hungarian
    "hy_am": ["hye-Armn"],  # Armenian
    "id_id": ["ind-Latn"],  # Indonesian
    "ig_ng": ["ibo-Latn"],  # Igbo
    "is_is": ["isl-Latn"],  # Icelandic
    "it_it": ["ita-Latn"],  # Italian
    "ja_jp": ["jpn-Jpan"],  # Japanese
    "jv_id": ["jav-Latn"],  # Javanese
    "ka_ge": ["kat-Geor"],  # Georgian
    "kam_ke": ["kam-Latn"],  # Kamba
    "kea_cv": ["kea-Latn"],  # Kabuverdianu
    "kk_kz": ["kaz-Cyrl"],  # Kazakh
    "km_kh": ["khm-Khmr"],  # Khmer
    "kn_in": ["kan-Knda"],  # Kannada
    "ko_kr": ["kor-Hang"],  # Korean
    "ckb_iq": ["ckb-Arab"],  # Sorani Kurdish
    "ky_kg": ["kir-Cyrl"],  # Kyrgyz
    "lb_lu": ["ltz-Latn"],  # Luxembourgish
    "lg_ug": ["lug-Latn"],  # Ganda
    "ln_cd": ["lin-Latn"],  # Lingala
    "lo_la": ["lao-Laoo"],  # Lao
    "lt_lt": ["lit-Latn"],  # Lithuanian
    "luo_ke": ["luo-Latn"],  # Luo
    "lv_lv": ["lvs-Latn"],  # Latvian (Standard)
    "mi_nz": ["mri-Latn"],  # Maori
    "mk_mk": ["mkd-Cyrl"],  # Macedonian
    "ml_in": ["mal-Mlym"],  # Malayalam
    "mn_mn": ["mon-Cyrl"],  # Mongolian
    "mr_in": ["mar-Deva"],  # Marathi
    "ms_my": ["msa-Latn"],  # Malay
    "mt_mt": ["mlt-Latn"],  # Maltese
    "my_mm": ["mya-Mymr"],  # Burmese
    "nb_no": ["nob-Latn"],  # Norwegian Bokmål
    "ne_np": ["npi-Deva"],  # Nepali
    "nl_nl": ["nld-Latn"],  # Dutch
    "nso_za": ["nso-Latn"],  # Northern Sotho
    "ny_mw": ["nya-Latn"],  # Nyanja
    "oc_fr": ["oci-Latn"],  # Occitan
    "om_et": ["orm-Latn"],  # Oromo
    "or_in": ["ori-Orya"],  # Odia (Oriya)
    "pa_in": ["pan-Guru"],  # Punjabi
    "pl_pl": ["pol-Latn"],  # Polish
    "ps_af": ["pus-Arab"],  # Pashto
    "pt_br": ["por-Latn"],  # Portuguese (Brazil)
    "ro_ro": ["ron-Latn"],  # Romanian
    "ru_ru": ["rus-Cyrl"],  # Russian
    "bg_bg": ["bul-Cyrl"],  # Bulgarian
    "sd_in": ["snd-Arab"],  # Sindhi
    "sk_sk": ["slk-Latn"],  # Slovak
    "sl_si": ["slv-Latn"],  # Slovenian
    "sn_zw": ["sna-Latn"],  # Shona
    "so_so": ["som-Latn"],  # Somali
    "sr_rs": ["srp-Cyrl"],  # Serbian
    "sv_se": ["swe-Latn"],  # Swedish
    "sw_ke": ["swh-Latn"],  # Swahili
    "ta_in": ["tam-Taml"],  # Tamil
    "te_in": ["tel-Telu"],  # Telugu
    "tg_tj": ["tgk-Cyrl"],  # Tajik
    "th_th": ["tha-Thai"],  # Thai
    "tr_tr": ["tur-Latn"],  # Turkish
    "uk_ua": ["ukr-Cyrl"],  # Ukrainian
    "umb_ao": ["umb-Latn"],  # Umbundu
    "ur_pk": ["urd-Arab"],  # Urdu
    "uz_uz": ["uzn-Latn"],  # Uzbek (Northern)
    "vi_vn": ["vie-Latn"],  # Vietnamese
    "wo_sn": ["wol-Latn"],  # Wolof
    "xh_za": ["xho-Latn"],  # Xhosa
    "yo_ng": ["yor-Latn"],  # Yoruba
    "zu_za": ["zul-Latn"],  # Zulu
}


class FleursA2TRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="FleursA2TRetrieval",
        description="Speech recordings with corresponding text transcriptions from the FLEURS dataset.",
        reference="https://github.com/google-research-datasets/fleurs",
        dataset={
            "path": "mteb/fleurs",
            "revision": "720cfffb024c8a4691f26a2b79d471b1f99a49a0",
        },
        type="Any2AnyRetrieval",
        category="a2t",
        modalities=["audio", "text"],
        eval_splits=["test"],
        eval_langs=_FLEURS_EVAL_LANGS,
        main_score="hit_rate_at_5",
        date=("2021-01-01", "2024-12-31"),
        domains=["Spoken"],
        task_subtypes=["Speech Transcription Retrieval"],
        license="apache-2.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{conneau2023fleurs,
  author = {Conneau, Alexis and Ma, Min and Khanuja, Simran and Zhang, Yu and Axelrod, Vera and Dalmia, Siddharth and Riesa, Jason and Rivera, Clara and Bapna, Ankur},
  booktitle = {2022 IEEE Spoken Language Technology Workshop (SLT)},
  organization = {IEEE},
  pages = {798--805},
  title = {Fleurs: Few-shot learning evaluation of universal representations of speech},
  year = {2023},
}
""",
    )

    def load_data(self, **kwargs):
        if self.data_loaded:
            return
        self.corpus = defaultdict(DatasetDict)
        self.queries = defaultdict(DatasetDict)
        self.relevant_docs = defaultdict(DatasetDict)
        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self, num_proc: int | None = None, **kwargs: Any) -> None:
        """A2T: Query = audio, Corpus = text."""
        for lang in tqdm(self.hf_subsets, desc="Loading FleursA2TRetrieval subsets"):
            lang_dataset = datasets.load_dataset(
                self.metadata.dataset["path"],
                lang,
                revision=self.metadata.dataset["revision"],
            )

            for split in self.metadata.eval_splits:
                split_dataset = lang_dataset[split]

                queries_ds = split_dataset.select_columns(["id", "audio"])
                corpus_ds = split_dataset.select_columns(
                    ["id", "transcription"]
                ).rename_column("transcription", "text")

                relevant_docs_ = {
                    str(row["id"]): {str(row["id"]): 1} for row in split_dataset
                }

                self.corpus[lang][split] = corpus_ds
                self.queries[lang][split] = queries_ds
                self.relevant_docs[lang][split] = relevant_docs_


class FleursT2ARetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="FleursT2ARetrieval",
        description="Speech recordings with corresponding text transcriptions from the FLEURS dataset.",
        reference="https://github.com/google-research-datasets/fleurs",
        dataset={
            "path": "mteb/fleurs",
            "revision": "720cfffb024c8a4691f26a2b79d471b1f99a49a0",
        },
        type="Any2AnyRetrieval",
        category="t2a",
        modalities=["text", "audio"],
        eval_splits=["test"],
        eval_langs=_FLEURS_EVAL_LANGS,
        main_score="hit_rate_at_5",
        date=("2021-01-01", "2024-12-31"),
        domains=["Spoken"],
        task_subtypes=["Speech Retrieval"],
        license="apache-2.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{conneau2023fleurs,
  author = {Conneau, Alexis and Ma, Min and Khanuja, Simran and Zhang, Yu and Axelrod, Vera and Dalmia, Siddharth and Riesa, Jason and Rivera, Clara and Bapna, Ankur},
  booktitle = {2022 IEEE Spoken Language Technology Workshop (SLT)},
  organization = {IEEE},
  pages = {798--805},
  title = {Fleurs: Few-shot learning evaluation of universal representations of speech},
  year = {2023},
}
""",
    )

    def load_data(self, **kwargs):
        if self.data_loaded:
            return
        self.corpus = defaultdict(DatasetDict)
        self.queries = defaultdict(DatasetDict)
        self.relevant_docs = defaultdict(DatasetDict)
        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self, num_proc: int | None = None, **kwargs: Any) -> None:
        """T2A: Query = text, Corpus = audio."""
        for lang in tqdm(self.hf_subsets, desc="Loading FleursT2ARetrieval subsets"):
            lang_dataset = datasets.load_dataset(
                self.metadata.dataset["path"],
                lang,
                revision=self.metadata.dataset["revision"],
            )

            for split in self.metadata.eval_splits:
                split_dataset = lang_dataset[split]

                # Create datasets directly without intermediate lists
                queries_ds = split_dataset.select_columns(
                    ["id", "transcription"]
                ).rename_column("transcription", "text")

                corpus_ds = split_dataset.select_columns(["id", "audio"])

                # Create relevant_docs mapping
                relevant_docs_ = {
                    str(row["id"]): {str(row["id"]): 1} for row in split_dataset
                }

                self.corpus[lang][split] = corpus_ds
                self.queries[lang][split] = queries_ds
                self.relevant_docs[lang][split] = relevant_docs_



from mteb.abstasks.sts import AbsTaskSTS
from mteb.abstasks.task_metadata import TaskMetadata

_LANGUAGES = {
    "ko-ko": ["kor-Hang"],
    "ar-ar": ["ara-Arab"],
    "en-ar": ["eng-Latn", "ara-Arab"],
    "en-de": ["eng-Latn", "deu-Latn"],
    "en-en": ["eng-Latn"],
    "en-tr": ["eng-Latn", "tur-Latn"],
    "es-en": ["spa-Latn", "eng-Latn"],
    "es-es": ["spa-Latn"],
    "fr-en": ["fra-Latn", "eng-Latn"],
    "it-en": ["ita-Latn", "eng-Latn"],
    "nl-en": ["nld-Latn", "eng-Latn"],
}


class STS17Crosslingual(AbsTaskSTS):
    fast_loading = True
    metadata = TaskMetadata(
        name="STS17",
        dataset={
            "path": "mteb/sts17-crosslingual-sts",
            "revision": "faeb762787bd10488a50c8b5be4a3b82e411949c",
        },
        description="Semeval-2017 task 1: Semantic textual similarity-multilingual and cross-lingual focused evaluation",
        reference="https://alt.qcri.org/semeval2017/task1/",
        type="STS",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=_LANGUAGES,
        main_score="cosine_spearman",
        date=("2014-01-01", "2015-12-31"),
        domains=["News", "Web", "Written"],
        task_subtypes=[],
        license="not specified",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="created",
        bibtex_citation=r"""
@inproceedings{cer-etal-2017-semeval,
  address = {Vancouver, Canada},
  author = {Cer, Daniel  and
Diab, Mona  and
Agirre, Eneko  and
Lopez-Gazpio, I{\\~n}igo  and
Specia, Lucia},
  booktitle = {Proceedings of the 11th International Workshop on Semantic Evaluation ({S}em{E}val-2017)},
  doi = {10.18653/v1/S17-2001},
  editor = {Bethard, Steven  and
Carpuat, Marine  and
Apidianaki, Marianna  and
Mohammad, Saif M.  and
Cer, Daniel  and
Jurgens, David},
  month = aug,
  pages = {1--14},
  publisher = {Association for Computational Linguistics},
  title = {{S}em{E}val-2017 Task 1: Semantic Textual Similarity Multilingual and Crosslingual Focused Evaluation},
  url = {https://aclanthology.org/S17-2001},
  year = {2017},
}
""",
    )

    min_score = 0
    max_score = 5



from __future__ import annotations

from typing import TYPE_CHECKING, Any

import torch
from packaging.specifiers import SpecifierSet
from torch.nn.functional import normalize
from torch.utils.data import DataLoader
from tqdm.auto import tqdm
from transformers import __version__ as transformers_version

from mteb._requires_package import requires_package
from mteb.models.abs_encoder import AbsEncoder
from mteb.models.model_meta import ModelMeta
from mteb.types import PromptType

if TYPE_CHECKING:
    from mteb.abstasks.task_metadata import TaskMetadata
    from mteb.types import Array, BatchedInput

LLAMA_NEMORETRIEVER_CITATION = """@misc{xu2025llamanemoretrievercolembedtopperforming,
      title={Llama Nemoretriever Colembed: Top-Performing Text-Image Retrieval Model},
      author={Mengyao Xu and Gabriel Moreira and Ronay Ak and Radek Osmulski and Yauhen Babakhin and Zhiding Yu and Benedikt Schifferer and Even Oldridge},
      year={2025},
      eprint={2507.05513},
      archivePrefix={arXiv},
      primaryClass={cs.CV},
      url={https://arxiv.org/abs/2507.05513}
}"""

NEMOTRON_COLEMBED_CITATION_V2 = """
@misc{moreira2026nemotroncolembedv2topperforming,
    title={Nemotron ColEmbed V2: Top-Performing Late Interaction embedding models for Visual Document Retrieval},
    author={Gabriel de Souza P. Moreira and Ronay Ak and Mengyao Xu and Oliver Holworthy and Benedikt Schifferer and Zhiding Yu and Yauhen Babakhin and Radek Osmulski and Jiarui Cai and Ryan Chesler and Bo Liu and Even Oldridge},
    year={2026},
    eprint={2602.03992},
    archivePrefix={arXiv},
    primaryClass={cs.IR},
    url={https://arxiv.org/abs/2602.03992},
}"""

NEMOTRON_EMBED_VL_1B_V2_CITATION = """
@misc{ronay2026smallyetmighty,
    title={Small Yet Mighty: Improve Accuracy In Multimodal Search and Visual Document Retrieval with Llama Nemotron RAG Models},
    author={Ronay Ak, Gabriel de Souza Pereira Moreira and Bo Liu},
    year={2026},
    howpublished = {Available at: https://huggingface.co/blog/nvidia/llama-nemotron-vl-1b},
}"""

# Transformers version constraints per extra.
# Keep in sync with pyproject.toml [project.optional-dependencies]
#
# Note: The extra name reflects the transformers version requirement, not the model version.
# For example, llama-nemotron-colembed-vl-3b-v2 uses "llama-nemotron-colembed-vl" because it
# requires transformers==4.49.0, even though it's a "v2" model by name.
_TRANSFORMERS_CONSTRAINTS: dict[str, str] = {
    "llama-nemotron-colembed-vl": "==4.49.0",  # llama-nemoretriever-colembed-*
    "nemotron-colembed-vl-v2": "==5.0.0",  # nemotron-colembed-vl-4b-v2, nemotron-colembed-vl-8b-v2
    "llama-nemotron-embed-vl-1b-v2": ">=4.56.0",  # llama-nemotron-embed-vl-1b-v2
}


class NemotronColEmbedVL(AbsEncoder):
    """Encoder for the NemotronColEmbedVL family of models."""

    def __init__(
        self,
        model_name_or_path: str,
        revision: str,
        trust_remote_code: bool,
        extra_name: str = "llama-nemotron-colembed-vl",
        device_map="cuda",
        torch_dtype=torch.bfloat16,
        attn_implementation="flash_attention_2",
        **kwargs,
    ):
        install_hint = f"pip install 'mteb[{extra_name}]'"

        # Check transformers version
        constraint = _TRANSFORMERS_CONSTRAINTS.get(extra_name)
        if constraint is None:
            raise ValueError(
                f"Unknown extra_name '{extra_name}'. "
                f"Must be one of: {list(_TRANSFORMERS_CONSTRAINTS.keys())}"
            )
        if transformers_version not in SpecifierSet(constraint):
            raise RuntimeError(
                f"Model `{model_name_or_path}` requires transformers{constraint}, "
                f"but {transformers_version} is installed. "
                f"Run: {install_hint}"
            )

        # Check required packages
        for package in ("torchvision", "accelerate", "flash_attn"):
            requires_package(self, package, model_name_or_path, install_hint)

        from transformers import AutoModel

        self.model = AutoModel.from_pretrained(
            model_name_or_path,
            revision=revision,
            device_map=device_map,
            trust_remote_code=trust_remote_code,
            torch_dtype=torch_dtype,
            attn_implementation=attn_implementation,
        ).eval()

    def get_text_embeddings(self, texts, batch_size: int = 32, **kwargs):
        return self.model.forward_queries(texts, batch_size=batch_size)

    def get_image_embeddings(
        self,
        images,
        batch_size: int = 32,
        **kwargs,
    ):
        import torchvision.transforms.functional as F
        from PIL import Image

        all_images = []
        if isinstance(images, DataLoader):
            iterator = images
        else:
            iterator = DataLoader(images, batch_size=batch_size)

        for batch in iterator:
            for image in batch["image"]:
                pil_img = (
                    image
                    if isinstance(image, Image.Image)
                    else F.to_pil_image(image.to("cpu"))
                )
                all_images.append(pil_img)

        return self.model.forward_images(all_images, batch_size=batch_size)

    def similarity(self, a, b):
        return self.model.get_scores(a, b)

    def get_fused_embeddings(
        self,
        *args,
        **kwargs,
    ):
        raise NotImplementedError(
            "Fused embeddings are not supported yet. Please use get_text_embeddings or get_image_embeddings."
        )

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        text_embeddings = None
        image_embeddings = None

        if "text" in inputs.dataset.features:
            text_embeddings = self.get_text_embeddings(inputs, **kwargs)
        if "image" in inputs.dataset.features:
            image_embeddings = self.get_image_embeddings(inputs, **kwargs)

        if text_embeddings is not None and image_embeddings is not None:
            raise NotImplementedError(
                "Fused embeddings are not supported yet. Please use get_text_embeddings or get_image_embeddings."
            )
        elif text_embeddings is not None:
            return text_embeddings
        elif image_embeddings is not None:
            return image_embeddings
        raise ValueError


TRAINING_DATA = {
    # from https://huggingface.co/datasets/vidore/colpali_train_set
    "VidoreDocVQARetrieval",
    "VidoreInfoVQARetrieval",
    "VidoreTatdqaRetrieval",
    "VidoreArxivQARetrieval",
    "HotpotQA",
    "MIRACLRetrieval",
    "NQ",
    "StackExchangeClustering",
    "SQuAD",
    "WebInstructSub",
    "docmatix-ir",
    "VDRMultilingualRetrieval",
    "VisRAG-Ret-Train-Synthetic-data",
    "VisRAG-Ret-Train-In-domain-data",
    "wiki-ss-nq",
}


TRAINING_DATA_v2 = {
    "VidoreDocVQARetrieval",
    "VidoreInfoVQARetrieval",
    "VidoreTatdqaRetrieval",
    "VidoreArxivQARetrieval",
    "docmatix-ir",
    "VDRMultilingualRetrieval",
    "VisRAG-Ret-Train-Synthetic-data",
    "VisRAG-Ret-Train-In-domain-data",
    "wiki-ss-nq",
}

llama_nemoretriever_colembed_1b_v1 = ModelMeta(
    loader=NemotronColEmbedVL,
    loader_kwargs=dict(
        extra_name="llama-nemotron-colembed-vl",
        trust_remote_code=True,
    ),
    name="nvidia/llama-nemoretriever-colembed-1b-v1",
    model_type=["late-interaction"],
    languages=["eng-Latn"],
    revision="6eade800103413033f260bb55b49fe039fd28a6e",
    release_date="2025-06-27",
    modalities=["image", "text"],
    n_parameters=2_418_000_000,
    n_embedding_parameters=262688768,
    memory_usage_mb=4610,
    max_tokens=8192,
    embed_dim=2048,
    license="https://huggingface.co/nvidia/llama-nemoretriever-colembed-1b-v1/blob/main/LICENSE",
    open_weights=True,
    public_training_code=None,
    public_training_data="https://huggingface.co/nvidia/llama-nemoretriever-colembed-1b-v1#training-dataset",
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/nvidia/llama-nemoretriever-colembed-1b-v1",
    similarity_fn_name="MaxSim",
    use_instructions=True,
    training_datasets=TRAINING_DATA,
    citation=LLAMA_NEMORETRIEVER_CITATION,
)

llama_nemoretriever_colembed_3b_v1 = ModelMeta(
    loader=NemotronColEmbedVL,
    loader_kwargs=dict(
        extra_name="llama-nemotron-colembed-vl",
        trust_remote_code=True,
    ),
    name="nvidia/llama-nemoretriever-colembed-3b-v1",
    model_type=["late-interaction"],
    languages=["eng-Latn"],
    revision="4194bdd2cd2871f220ddba6273ce173ef1217a1e",
    release_date="2025-06-27",
    modalities=["image", "text"],
    n_parameters=4_407_000_000,
    n_embedding_parameters=394033152,
    memory_usage_mb=8403,
    max_tokens=8192,
    embed_dim=3072,
    license="https://huggingface.co/nvidia/llama-nemoretriever-colembed-1b-v1/blob/main/LICENSE",
    open_weights=True,
    public_training_code=None,
    public_training_data="https://huggingface.co/nvidia/llama-nemoretriever-colembed-1b-v1#training-dataset",
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/nvidia/llama-nemoretriever-colembed-3b-v1",
    similarity_fn_name="MaxSim",
    use_instructions=True,
    training_datasets=TRAINING_DATA,
    citation=LLAMA_NEMORETRIEVER_CITATION,
)

llama_nemotron_colembed_vl_3b_v2 = ModelMeta(
    loader=NemotronColEmbedVL,
    loader_kwargs=dict(
        extra_name="llama-nemotron-colembed-vl",
        trust_remote_code=True,
    ),
    name="nvidia/llama-nemotron-colembed-vl-3b-v2",
    model_type=["late-interaction"],
    languages=["eng-Latn"],
    revision="680b47b199f99bc0ec2f4e90ffa583ec0c2e452c",
    release_date="2026-01-21",
    modalities=["image", "text"],
    n_parameters=4_407_000_000,
    n_embedding_parameters=394033152,
    memory_usage_mb=8403,
    max_tokens=8192,
    embed_dim=3072,
    license="https://huggingface.co/nvidia/llama-nemotron-colembed-vl-3b-v2/blob/main/LICENSE",
    open_weights=True,
    public_training_code=None,
    public_training_data="https://huggingface.co/nvidia/llama-nemotron-colembed-vl-3b-v2#training-dataset",
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/nvidia/llama-nemotron-colembed-vl-3b-v2",
    similarity_fn_name="MaxSim",
    use_instructions=True,
    training_datasets=TRAINING_DATA,
    citation=NEMOTRON_COLEMBED_CITATION_V2,
)


nemotron_colembed_vl_4b_v2 = ModelMeta(
    loader=NemotronColEmbedVL,
    loader_kwargs=dict(
        extra_name="nemotron-colembed-vl-v2",
        trust_remote_code=True,
    ),
    name="nvidia/nemotron-colembed-vl-4b-v2",
    revision="0ed152d91f8ad4c5d48296b51c220f686641a398",
    languages=["eng-Latn"],
    release_date="2026-01-07",
    modalities=["image", "text"],
    n_parameters=4_800_000_000,
    n_embedding_parameters=388956160,
    memory_usage_mb=9206,
    max_tokens=262144,
    embed_dim=2560,
    license="https://huggingface.co/nvidia/nemotron-colembed-vl-4b-v2/blob/main/LICENSE",
    open_weights=True,
    public_training_code=None,
    public_training_data="https://huggingface.co/nvidia/nemotron-colembed-vl-4b-v2#training-dataset",
    framework=["PyTorch", "Transformers"],
    reference="https://huggingface.co/nvidia/nemotron-colembed-vl-4b-v2",
    similarity_fn_name="MaxSim",
    use_instructions=True,
    training_datasets=TRAINING_DATA_v2,
    citation=NEMOTRON_COLEMBED_CITATION_V2,
    model_type=["late-interaction"],
)


nemotron_colembed_vl_8b_v2 = ModelMeta(
    loader=NemotronColEmbedVL,
    loader_kwargs=dict(
        extra_name="nemotron-colembed-vl-v2",
        trust_remote_code=True,
    ),
    name="nvidia/nemotron-colembed-vl-8b-v2",
    revision="34b640612f311ed05a6c7c62c6564847ed555f5f",
    languages=["eng-Latn"],
    release_date="2026-01-07",
    modalities=["image", "text"],
    n_parameters=8_700_000_000,
    n_embedding_parameters=622329856,
    memory_usage_mb=16722,
    max_tokens=262144,
    embed_dim=4096,
    license="https://huggingface.co/nvidia/nemotron-colembed-vl-8b-v2/blob/main/LICENSE",
    open_weights=True,
    public_training_code=None,
    public_training_data="https://huggingface.co/nvidia/nemotron-colembed-vl-8b-v2#training-dataset",
    framework=["PyTorch", "Transformers"],
    reference="https://huggingface.co/nvidia/nemotron-colembed-vl-8b-v2",
    similarity_fn_name="MaxSim",
    use_instructions=True,
    training_datasets=TRAINING_DATA_v2,
    citation=NEMOTRON_COLEMBED_CITATION_V2,
    model_type=["late-interaction"],
)


class LlamaNemotronEmbedVL(AbsEncoder):
    def __init__(
        self,
        model_name_or_path: str,
        revision: str,
        trust_remote_code: bool,
        extra_name: str = "llama-nemotron-embed-vl-1b-v2",
        device_map="cuda",
        torch_dtype=torch.bfloat16,
        attn_implementation="flash_attention_2",
        use_image_modality: bool = True,
        use_text_modality: bool = True,
        **kwargs,
    ):
        self.use_image_modality = use_image_modality
        self.use_text_modality = use_text_modality
        if not self.use_image_modality and not self.use_text_modality:
            raise ValueError(
                "At least one of use_image_modality or use_text_modality must be True"
            )

        install_hint = f"pip install 'mteb[{extra_name}]'"

        # Check transformers version
        constraint = _TRANSFORMERS_CONSTRAINTS.get(extra_name)
        if constraint is None:
            raise ValueError(
                f"Unknown extra_name '{extra_name}'. "
                f"Must be one of: {list(_TRANSFORMERS_CONSTRAINTS.keys())}"
            )
        if transformers_version not in SpecifierSet(constraint):
            raise RuntimeError(
                f"Model `{model_name_or_path}` requires transformers{constraint}, "
                f"but {transformers_version} is installed. "
                f"Run: {install_hint}"
            )

        # Check required packages
        for package in ("torchvision", "accelerate", "flash_attn"):
            requires_package(self, package, model_name_or_path, install_hint)

        from transformers import AutoModel

        self.model = AutoModel.from_pretrained(
            model_name_or_path,
            revision=revision,
            device_map=device_map,
            trust_remote_code=trust_remote_code,
            torch_dtype=torch_dtype,
            attn_implementation=attn_implementation,
        ).eval()

        # Sets the number of tiles the image can be split into
        self.model.processor.max_input_tiles = 4

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        show_progress_bar: bool = True,
        **kwargs: Any,
    ) -> Array:
        with torch.inference_mode():
            embeddings_list = []
            for batch in tqdm(
                inputs,
                desc=f"Extracting {prompt_type} embeddings...",
                disable=not show_progress_bar,
            ):
                if prompt_type == PromptType.query and "text" in batch:
                    embeddings = self.model.encode_queries(batch["text"])
                else:
                    if not self.use_image_modality and "image" in batch:
                        del batch["image"]
                    if not self.use_text_modality and "text" in batch:
                        del batch["text"]

                    if "image" in batch and "text" in batch:
                        embeddings = self.model.encode_documents(
                            images=batch["image"], texts=batch["text"]
                        )
                    elif "image" in batch:
                        embeddings = self.model.encode_documents(images=batch["image"])
                    elif "text" in batch:
                        embeddings = self.model.encode_documents(texts=batch["text"])
                    else:
                        raise ValueError(
                            f"Could not find 'image' or 'text' in batch: {batch}"
                        )

                embeddings = normalize(embeddings, dim=-1)
                assert torch.sum(embeddings).float().item() not in [
                    0.0,
                    float("inf"),
                ]
                embeddings_list.append(embeddings)

            concatenated_embeddings = torch.vstack(embeddings_list)
            return concatenated_embeddings


TRAINING_DATA_EMBED_VL_1B_V2 = {
    "VidoreDocVQARetrieval",
    "VidoreInfoVQARetrieval",
    "VidoreTatdqaRetrieval",
    "VidoreArxivQARetrieval",
    "docmatix-ir",
    "wiki-ss-nq",
    "Cauldron (AI2D, OCRVQA, Websight)",
    "VDRMultilingualRetrieval",
    "HotpotQA",
    "MIRACLRetrieval",
    "NQ",
    "StackExchangeClustering",
    "SQuAD",
    "MultiLongDocRetrieval",
    "MLQARetrieval",
    "Tiger Math/Stack",
}

llama_nemotron_embed_vl_1b_v2 = ModelMeta(
    loader=LlamaNemotronEmbedVL,
    loader_kwargs=dict(
        trust_remote_code=True,
    ),
    name="nvidia/llama-nemotron-embed-vl-1b-v2",
    languages=["eng-Latn"],
    revision="859e1f2dac29c56c37a5279cf55f53f3e74efc6b",
    release_date="2026-01-06",
    modalities=["image", "text"],
    n_parameters=1_678_252_480,
    n_embedding_parameters=262_688_768,
    memory_usage_mb=6402,
    max_tokens=10240,
    embed_dim=2048,
    license="https://www.nvidia.com/en-us/agreements/enterprise-software/nvidia-open-model-license/",
    open_weights=True,
    public_training_code=None,
    public_training_data="https://huggingface.co/nvidia/llama-nemotron-embed-vl-1b-v2#training-dataset",
    framework=["PyTorch"],
    reference="https://huggingface.co/nvidia/llama-nemotron-embed-vl-1b-v2",
    similarity_fn_name="cosine",
    use_instructions=True,
    training_datasets=TRAINING_DATA_EMBED_VL_1B_V2,
    citation=NEMOTRON_EMBED_VL_1B_V2_CITATION,
)



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata

_LANGUAGES = {
    "mlqa.ar.ar": ["ara-Arab", "ara-Arab"],
    "mlqa.ar.de": ["ara-Arab", "deu-Latn"],
    "mlqa.ar.en": ["ara-Arab", "eng-Latn"],
    "mlqa.ar.es": ["ara-Arab", "spa-Latn"],
    "mlqa.ar.hi": ["ara-Arab", "hin-Deva"],
    "mlqa.ar.vi": ["ara-Arab", "vie-Latn"],
    "mlqa.ar.zh": ["ara-Arab", "zho-Hans"],
    "mlqa.de.ar": ["deu-Latn", "ara-Arab"],
    "mlqa.de.de": ["deu-Latn", "deu-Latn"],
    "mlqa.de.en": ["deu-Latn", "eng-Latn"],
    "mlqa.de.es": ["deu-Latn", "spa-Latn"],
    "mlqa.de.hi": ["deu-Latn", "hin-Deva"],
    "mlqa.de.vi": ["deu-Latn", "vie-Latn"],
    "mlqa.de.zh": ["deu-Latn", "zho-Hans"],
    "mlqa.en.ar": ["eng-Latn", "ara-Arab"],
    "mlqa.en.de": ["eng-Latn", "deu-Latn"],
    "mlqa.en.en": ["eng-Latn", "eng-Latn"],
    "mlqa.en.es": ["eng-Latn", "spa-Latn"],
    "mlqa.en.hi": ["eng-Latn", "hin-Deva"],
    "mlqa.en.vi": ["eng-Latn", "vie-Latn"],
    "mlqa.en.zh": ["eng-Latn", "zho-Hans"],
    "mlqa.es.ar": ["spa-Latn", "ara-Arab"],
    "mlqa.es.de": ["spa-Latn", "deu-Latn"],
    "mlqa.es.en": ["spa-Latn", "eng-Latn"],
    "mlqa.es.es": ["spa-Latn", "spa-Latn"],
    "mlqa.es.hi": ["spa-Latn", "hin-Deva"],
    "mlqa.es.vi": ["spa-Latn", "vie-Latn"],
    "mlqa.es.zh": ["spa-Latn", "zho-Hans"],
    "mlqa.hi.ar": ["hin-Deva", "ara-Arab"],
    "mlqa.hi.de": ["hin-Deva", "deu-Latn"],
    "mlqa.hi.en": ["hin-Deva", "eng-Latn"],
    "mlqa.hi.es": ["hin-Deva", "spa-Latn"],
    "mlqa.hi.hi": ["hin-Deva", "hin-Deva"],
    "mlqa.hi.vi": ["hin-Deva", "vie-Latn"],
    "mlqa.hi.zh": ["hin-Deva", "zho-Hans"],
    "mlqa.vi.ar": ["vie-Latn", "ara-Arab"],
    "mlqa.vi.de": ["vie-Latn", "deu-Latn"],
    "mlqa.vi.en": ["vie-Latn", "eng-Latn"],
    "mlqa.vi.es": ["vie-Latn", "spa-Latn"],
    "mlqa.vi.hi": ["vie-Latn", "hin-Deva"],
    "mlqa.vi.vi": ["vie-Latn", "vie-Latn"],
    "mlqa.vi.zh": ["vie-Latn", "zho-Hans"],
    "mlqa.zh.ar": ["zho-Hans", "ara-Arab"],
    "mlqa.zh.de": ["zho-Hans", "deu-Latn"],
    "mlqa.zh.en": ["zho-Hans", "eng-Latn"],
    "mlqa.zh.es": ["zho-Hans", "spa-Latn"],
    "mlqa.zh.hi": ["zho-Hans", "hin-Deva"],
    "mlqa.zh.vi": ["zho-Hans", "vie-Latn"],
    "mlqa.zh.zh": ["zho-Hans", "zho-Hans"],
}


def _build_lang_pair(langs: list[str]) -> str:
    """Builds a language pair separated by a dash.
    e.g., ['eng-Latn', 'deu-Latn'] -> 'eng-deu'.
    """
    return langs[0].split("-")[0] + "-" + langs[1].split("-")[0]


def extend_lang_pairs() -> dict[str, list[str]]:
    eval_langs = {}
    for langs in _LANGUAGES.values():
        lang_pair = _build_lang_pair(langs)
        eval_langs[lang_pair] = langs
    return eval_langs


_EVAL_LANGS = extend_lang_pairs()


class MLQARetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="MLQARetrieval",
        description="MLQA (MultiLingual Question Answering) is a benchmark dataset for evaluating cross-lingual question answering performance. MLQA consists of over 5K extractive QA instances (12K in English) in SQuAD format in seven languages - English, Arabic, German, Spanish, Hindi, Vietnamese and Simplified Chinese. MLQA is highly parallel, with QA instances parallel between 4 different languages on average.",
        reference="https://huggingface.co/datasets/mlqa",
        dataset={
            "path": "mteb/MLQARetrieval",
            "revision": "5bef8b6e2601af974fb1a1cca03fd702229be4b6",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["validation", "test"],
        eval_langs=_EVAL_LANGS,
        main_score="ndcg_at_10",
        date=("2019-01-01", "2020-12-31"),
        domains=["Encyclopaedic", "Written"],
        task_subtypes=["Question answering"],
        license="cc-by-sa-3.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@article{lewis2019mlqa,
  author = {Lewis, Patrick and Oguz, Barlas and Rinott, Ruty and Riedel, Sebastian and Schwenk, Holger},
  eid = {arXiv: 1910.07475},
  journal = {arXiv preprint arXiv:1910.07475},
  title = {MLQA: Evaluating Cross-lingual Extractive Question Answering},
  year = {2019},
}
""",
    )



from __future__ import annotations

import logging
from typing import TYPE_CHECKING, Any

import torch
from tqdm.auto import tqdm

from mteb._requires_package import (
    requires_image_dependencies,
    requires_package,
)
from mteb.models.abs_encoder import AbsEncoder
from mteb.models.model_meta import ModelMeta, ScoringFunction

if TYPE_CHECKING:
    from torch.utils.data import DataLoader

    from mteb.abstasks.task_metadata import TaskMetadata
    from mteb.types import Array, BatchedInput, PromptType

from .colpali_models import (
    COLPALI_CITATION,
    COLPALI_TRAINING_DATA,
    ColPaliEngineWrapper,
)

logger = logging.getLogger(__name__)


class ColQwen2Wrapper(ColPaliEngineWrapper):
    """Wrapper for ColQwen2 model."""

    def __init__(
        self,
        model_name: str = "vidore/colqwen2-v1.0",
        revision: str | None = None,
        device: str | None = None,
        **kwargs,
    ):
        requires_package(
            self, "colpali_engine", model_name, "pip install mteb[colpali_engine]"
        )
        from colpali_engine.models import ColQwen2, ColQwen2Processor

        super().__init__(
            model_name=model_name,
            model_class=ColQwen2,
            processor_class=ColQwen2Processor,
            revision=revision,
            device=device,
            **kwargs,
        )


class ColQwen2_5Wrapper(ColPaliEngineWrapper):  # noqa: N801
    """Wrapper for ColQwen2.5 model."""

    def __init__(
        self,
        model_name: str = "vidore/colqwen2.5-v0.2",
        revision: str | None = None,
        device: str | None = None,
        attn_implementation: str | None = None,
        **kwargs,
    ):
        requires_package(
            self, "colpali_engine", model_name, "pip install mteb[colpali_engine]"
        )
        from colpali_engine.models import ColQwen2_5, ColQwen2_5_Processor
        from transformers.utils.import_utils import is_flash_attn_2_available

        if attn_implementation is None:
            attn_implementation = (
                "flash_attention_2" if is_flash_attn_2_available() else None
            )

        super().__init__(
            model_name=model_name,
            model_class=ColQwen2_5,
            processor_class=ColQwen2_5_Processor,
            revision=revision,
            device=device,
            **kwargs,
        )


class ColQwen3_5Wrapper(AbsEncoder):  # noqa: N801
    """Wrapper for ColQwen3.5 models (colpali_engine)."""

    def __init__(
        self,
        model_name: str = "athrael-soju/colqwen3.5-4.5B-v3",
        revision: str | None = None,
        device: str | None = None,
        **kwargs,
    ):
        requires_image_dependencies()
        requires_package(
            self, "colpali_engine", model_name, "pip install mteb[colpali_engine]"
        )
        from colpali_engine.models import ColQwen3_5, ColQwen3_5Processor

        self.device = device or ("cuda" if torch.cuda.is_available() else "cpu")

        self.model = ColQwen3_5.from_pretrained(
            model_name,
            device_map=self.device,
            adapter_kwargs={"revision": revision},
            **kwargs,
        )
        self.model.eval()

        self.processor = ColQwen3_5Processor.from_pretrained(model_name)

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        if (
            "text" not in inputs.dataset.features
            and "image" not in inputs.dataset.features
        ):
            raise ValueError("No text or image features found in inputs.")
        return self.get_fused_embeddings(inputs, **kwargs)

    def _encode_inputs(self, encoded_inputs: dict[str, torch.Tensor]) -> torch.Tensor:
        # Clear stale rope_deltas cache to avoid shape mismatches across batches
        if hasattr(self.model, "rope_deltas"):
            self.model.rope_deltas = None
        # ColQwen3_5.forward returns the projection tensor directly (not a named tuple)
        return self.model(**encoded_inputs)

    def get_fused_embeddings(
        self,
        image_texts_pairs: DataLoader[BatchedInput] | None = None,
        batch_size: int = 32,
        show_progress_bar: bool = True,
        **kwargs: Any,
    ):
        import torchvision.transforms.functional as F
        from PIL import Image

        contains_image = "image" in image_texts_pairs.dataset.features
        contains_text = "text" in image_texts_pairs.dataset.features
        contains_both = contains_image and contains_text

        if contains_both:
            progress_desc = "Encoding images+texts"
        elif contains_image:
            progress_desc = "Encoding images"
        elif contains_text:
            progress_desc = "Encoding texts"
        else:
            raise ValueError("No text or image features found in inputs.")

        all_embeds: list[torch.Tensor] = []
        with torch.no_grad():
            for batch in tqdm(
                image_texts_pairs,
                disable=not show_progress_bar,
                desc=progress_desc,
            ):
                if contains_image:
                    imgs = [
                        F.to_pil_image(b.to(self.device))
                        if not isinstance(b, Image.Image)
                        else b
                        for b in batch["image"]
                    ]
                else:
                    imgs = None
                if contains_text:
                    texts = batch["text"]
                else:
                    texts = None
                if contains_both:
                    assert len(imgs) == len(texts), (
                        f"The number of texts and images must have the same length, got {len(imgs)} and {len(texts)}"
                    )

                if contains_image:
                    imgs = [img.convert("RGB") for img in imgs]
                    inputs = self.processor.process_images(imgs)
                else:
                    inputs = self.processor.process_queries(texts)
                inputs = {k: v.to(self.device) for k, v in inputs.items()}
                outs = self._encode_inputs(inputs)
                all_embeds.extend(outs.cpu().to(torch.float32))

        padded = torch.nn.utils.rnn.pad_sequence(
            all_embeds, batch_first=True, padding_value=0
        )
        return padded

    def similarity(self, a, b):
        a = [torch.as_tensor(x) for x in a]
        b = [torch.as_tensor(x) for x in b]
        return self.processor.score_multi_vector(a, b, device=self.device)


class ColQwen3Wrapper(AbsEncoder):
    """Wrapper for the ColQwen3 vision-language retrieval model."""

    def __init__(
        self,
        model_name: str,
        *,
        revision: str | None = None,
        device: str | None = None,
        dtype: torch.dtype | str | None = torch.bfloat16,
        **kwargs: Any,
    ):
        requires_image_dependencies()
        requires_package(self, "transformers", model_name, "pip install mteb[colqwen3]")
        from transformers import AutoModel, AutoProcessor

        self.device = device or (
            "cuda"
            if torch.cuda.is_available()
            else "mps"
            if torch.backends.mps.is_available()
            else "cpu"
        )
        self.model = AutoModel.from_pretrained(
            model_name,
            revision=revision,
            dtype=dtype,
            trust_remote_code=True,
            **kwargs,
        ).to(self.device)
        self.model.eval()

        self.processor = AutoProcessor.from_pretrained(
            model_name,
            revision=revision,
            trust_remote_code=True,
            max_num_visual_tokens=1280,
        )

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        if (
            "text" not in inputs.dataset.features
            and "image" not in inputs.dataset.features
        ):
            raise ValueError("No text or image features found in inputs.")
        return self.get_fused_embeddings(inputs, **kwargs)

    def _encode_inputs(self, encoded_inputs: dict[str, torch.Tensor]) -> torch.Tensor:
        outputs = self.model(**encoded_inputs)
        # Avoid boolean casting of tensors when checking for custom attributes.
        embeddings = getattr(outputs, "embeddings", None)
        if embeddings is None:
            embeddings = outputs[0]
        return embeddings

    def get_fused_embeddings(
        self,
        image_texts_pairs: DataLoader[BatchedInput] | None = None,
        batch_size: int = 32,
        show_progress_bar: bool = True,
        fusion_mode="concat",
        **kwargs: Any,
    ):
        import torchvision.transforms.functional as F
        from PIL import Image

        contains_image = "image" in image_texts_pairs.dataset.features
        contains_text = "text" in image_texts_pairs.dataset.features
        contains_both = contains_image and contains_text

        if contains_both:
            progress_desc = "Encoding images+texts"
        elif contains_image:
            progress_desc = "Encoding images"
        elif contains_text:
            progress_desc = "Encoding texts"
        else:
            raise ValueError("No text or image features found in inputs.")

        all_embeds: list[torch.Tensor] = []
        with torch.no_grad():
            for batch in tqdm(
                image_texts_pairs,
                disable=not show_progress_bar,
                desc=progress_desc,
            ):
                if contains_image:
                    imgs = [
                        F.to_pil_image(b.to(self.device))
                        if not isinstance(b, Image.Image)
                        else b
                        for b in batch["image"]
                    ]
                else:
                    imgs = None
                if contains_text:
                    texts = batch["text"]
                else:
                    texts = None
                if contains_both:
                    assert len(imgs) == len(texts), (
                        f"The number of texts and images must have the same length, got {len(imgs)} and {len(texts)}"
                    )

                inputs = self.processor(images=imgs, text=texts)
                inputs = {k: v.to(self.device) for k, v in inputs.items()}
                outs = self._encode_inputs(inputs)
                all_embeds.extend(outs.cpu().to(torch.float32))

        padded = torch.nn.utils.rnn.pad_sequence(
            all_embeds, batch_first=True, padding_value=0
        )
        return padded

    def similarity(self, a, b):
        return self.processor.score_multi_vector(a, b, device=self.device)


colqwen2 = ModelMeta(
    loader=ColQwen2Wrapper,
    loader_kwargs=dict(
        torch_dtype=torch.float16,
    ),
    name="vidore/colqwen2-v1.0",
    model_type=["late-interaction"],
    languages=["eng-Latn"],
    revision="530094e83a40ca4edcb5c9e5ddfa61a4b5ea0d2f",
    release_date="2025-11-03",
    modalities=["image", "text"],
    n_parameters=2_210_000_000,
    n_embedding_parameters=None,
    memory_usage_mb=7200,
    max_tokens=32768,
    embed_dim=128,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/illuin-tech/colpali",
    public_training_data="https://huggingface.co/datasets/vidore/colpali_train_set",
    framework=["ColPali", "safetensors"],
    reference="https://huggingface.co/vidore/colqwen2-v1.0",
    similarity_fn_name="MaxSim",
    use_instructions=True,
    training_datasets=COLPALI_TRAINING_DATA,
    citation=COLPALI_CITATION,
)

colqwen2_5 = ModelMeta(
    loader=ColQwen2_5Wrapper,
    loader_kwargs=dict(
        torch_dtype=torch.float16,
    ),
    name="vidore/colqwen2.5-v0.2",
    model_type=["late-interaction"],
    languages=["eng-Latn"],
    revision="6f6fcdfd1a114dfe365f529701b33d66b9349014",
    release_date="2025-01-31",
    modalities=["image", "text"],
    n_parameters=3_000_000_000,
    n_embedding_parameters=None,
    memory_usage_mb=7200,
    max_tokens=128000,
    embed_dim=128,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/illuin-tech/colpali",
    public_training_data="https://huggingface.co/datasets/vidore/colpali_train_set",
    framework=["ColPali", "safetensors"],
    reference="https://huggingface.co/vidore/colqwen2.5-v0.2",
    similarity_fn_name="MaxSim",
    use_instructions=True,
    training_datasets=COLPALI_TRAINING_DATA,
    citation=COLPALI_CITATION,
)

TOMORO_TRAINING_DATA = {
    "VDRMultilingualRetrieval",
    # from https://huggingface.co/datasets/vidore/colpali_train_set
    "VidoreDocVQARetrieval",
    "VidoreInfoVQARetrieval",
    "VidoreTatdqaRetrieval",
    "VidoreArxivQARetrieval",
    "VisRAG-Ret-Train-Synthetic-data",
    "VisRAG-Ret-Train-In-domain-data",
}

TOMORO_CITATION = """
@misc{huang2025tomoro_colqwen3_embed,
  title={TomoroAI/tomoro-colqwen3-embed},
  author={Xin Huang and Kye Min Tan and Albert Phelps},
  year={2025},
  url={https://huggingface.co/TomoroAI/tomoro-colqwen3-embed-8b}
}
"""

colqwen3_8b = ModelMeta(
    loader=ColQwen3Wrapper,
    name="TomoroAI/tomoro-colqwen3-embed-8b",
    model_type=["late-interaction"],
    languages=["eng-Latn"],
    revision="0b9fe28142910e209bbac15b1efe85507c27644f",
    release_date="2025-11-26",
    modalities=["image", "text"],
    n_parameters=8_000_000_000,
    n_embedding_parameters=None,
    memory_usage_mb=16724,
    max_tokens=262144,
    embed_dim=320,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/illuin-tech/colpali",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/TomoroAI/tomoro-colqwen3-embed-8b",
    similarity_fn_name=ScoringFunction.MAX_SIM,
    use_instructions=True,
    training_datasets=TOMORO_TRAINING_DATA,
    citation=TOMORO_CITATION,
)

colqwen3_4b = ModelMeta(
    loader=ColQwen3Wrapper,
    name="TomoroAI/tomoro-colqwen3-embed-4b",
    model_type=["late-interaction"],
    languages=["eng-Latn"],
    revision="6a32fb68598730bf5620fbf18d832c784235c59c",
    release_date="2025-11-26",
    modalities=["image", "text"],
    n_parameters=4_000_000_000,
    n_embedding_parameters=None,
    memory_usage_mb=8466,
    max_tokens=262144,
    embed_dim=320,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/illuin-tech/colpali",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/TomoroAI/tomoro-colqwen3-embed-4b",
    similarity_fn_name=ScoringFunction.MAX_SIM,
    use_instructions=True,
    training_datasets=TOMORO_TRAINING_DATA,
    citation=TOMORO_CITATION,
)


COLNOMIC_CITATION = """
@misc{nomicembedmultimodal2025,
  title={Nomic Embed Multimodal: Interleaved Text, Image, and Screenshots for Visual Document Retrieval},
  author={Nomic Team},
  year={2025},
  publisher={Nomic AI},
  url={https://nomic.ai/blog/posts/nomic-embed-multimodal}
}"""

COLNOMIC_TRAINING_DATA = {"VDRMultilingual"} | COLPALI_TRAINING_DATA
COLNOMIC_LANGUAGES = [
    "deu-Latn",  # German
    "spa-Latn",  # Spanish
    "eng-Latn",  # English
    "fra-Latn",  # French
    "ita-Latn",  # Italian
]

colnomic_3b = ModelMeta(
    loader=ColQwen2_5Wrapper,
    loader_kwargs=dict(
        torch_dtype=torch.float16, attn_implementation="flash_attention_2"
    ),
    name="nomic-ai/colnomic-embed-multimodal-3b",
    model_type=["late-interaction"],
    languages=COLNOMIC_LANGUAGES,
    revision="86627b4a9b0cade577851a70afa469084f9863a4",
    release_date="2025-03-31",
    modalities=["image", "text"],
    n_parameters=3_000_000_000,
    n_embedding_parameters=None,
    memory_usage_mb=7200,
    max_tokens=128000,
    embed_dim=128,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/nomic-ai/colpali",
    public_training_data="https://huggingface.co/datasets/vidore/colpali_train_set",
    framework=["ColPali", "safetensors"],
    reference="https://huggingface.co/nomic-ai/colnomic-embed-multimodal-3b",
    similarity_fn_name="MaxSim",
    use_instructions=True,
    training_datasets=COLNOMIC_TRAINING_DATA,
    citation=COLNOMIC_CITATION,
)

colnomic_7b = ModelMeta(
    loader=ColQwen2_5Wrapper,
    loader_kwargs=dict(
        torch_dtype=torch.float16,
    ),
    name="nomic-ai/colnomic-embed-multimodal-7b",
    model_type=["late-interaction"],
    languages=COLNOMIC_LANGUAGES,
    revision="09dbc9502b66605d5be56d2226019b49c9fd3293",
    release_date="2025-03-31",
    modalities=["image", "text"],
    n_parameters=7_000_000_000,
    memory_usage_mb=14400,
    max_tokens=128000,
    embed_dim=128,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/nomic-ai/colpali",
    public_training_data="https://huggingface.co/datasets/vidore/colpali_train_set",
    framework=["ColPali"],
    reference="https://huggingface.co/nomic-ai/colnomic-embed-multimodal-7b",
    similarity_fn_name="MaxSim",
    use_instructions=True,
    training_datasets=COLNOMIC_TRAINING_DATA,
    citation=COLNOMIC_CITATION,
)


EVOQWEN_TRAINING_DATA = {
    # "colpali_train_set",
    "VidoreDocVQARetrieval",
    "VidoreInfoVQARetrieval",
    "VidoreTatdqaRetrieval",
    "VidoreArxivQARetrieval",
    "VisRAG-Ret-Train-Synthetic-data",
    "VisRAG-Ret-Train-In-domain-data",
}

evoqwen25_vl_retriever_3b_v1 = ModelMeta(
    loader=ColQwen2_5Wrapper,
    loader_kwargs=dict(
        torch_dtype=torch.float16, attn_implementation="flash_attention_2"
    ),
    name="ApsaraStackMaaS/EvoQwen2.5-VL-Retriever-3B-v1",
    model_type=["late-interaction"],
    languages=["eng-Latn"],
    revision="aeacaa2775f2758d82721eb1cf2f5daf1a392da9",
    release_date="2025-11-04",
    modalities=["image", "text"],
    n_parameters=3_000_000_000,
    n_embedding_parameters=None,
    memory_usage_mb=7200,
    max_tokens=128000,
    embed_dim=128,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/illuin-tech/colpali",
    public_training_data="https://huggingface.co/datasets/vidore/colpali_train_set",
    framework=["ColPali", "safetensors"],
    reference="https://huggingface.co/ApsaraStackMaaS/EvoQwen2.5-VL-Retriever-3B-v1",
    similarity_fn_name="MaxSim",
    use_instructions=True,
    training_datasets=EVOQWEN_TRAINING_DATA,
)

evoqwen25_vl_retriever_7b_v1 = ModelMeta(
    loader=ColQwen2_5Wrapper,
    loader_kwargs=dict(
        torch_dtype=torch.float16, attn_implementation="flash_attention_2"
    ),
    name="ApsaraStackMaaS/EvoQwen2.5-VL-Retriever-7B-v1",
    model_type=["late-interaction"],
    languages=["eng-Latn"],
    revision="8952ac6ee0e7de2e9211b165921518caf9202110",
    release_date="2025-11-04",
    modalities=["image", "text"],
    n_parameters=7_000_000_000,
    n_embedding_parameters=None,
    memory_usage_mb=14400,
    max_tokens=128000,
    embed_dim=128,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/illuin-tech/colpali",
    public_training_data="https://huggingface.co/datasets/vidore/colpali_train_set",
    framework=["ColPali", "safetensors"],
    reference="https://huggingface.co/ApsaraStackMaaS/EvoQwen2.5-VL-Retriever-7B-v1",
    similarity_fn_name="MaxSim",
    use_instructions=True,
    training_datasets=EVOQWEN_TRAINING_DATA,
)


COLQWEN35_V3_TRAINING_DATA = {
    # from https://huggingface.co/datasets/vidore/colpali_train_set
    "VidoreDocVQARetrieval",
    "VidoreInfoVQARetrieval",
    "VidoreTatdqaRetrieval",
    "VidoreArxivQARetrieval",
    # from https://huggingface.co/datasets/openbmb/VisRAG-Ret-Train-Synthetic-data
    "VisRAG-Ret-Train-Synthetic-data",
    # from https://huggingface.co/datasets/openbmb/VisRAG-Ret-Train-In-domain-data
    "VisRAG-Ret-Train-In-domain-data",
    # from https://huggingface.co/datasets/llamaindex/vdr-multilingual-train
    "VDRMultilingualRetrieval",
    # from https://huggingface.co/datasets/Metric-AI/tabfquad_train_set
    "VidoreTabfquadRetrieval",
}

colqwen3_5_v3 = ModelMeta(
    loader=ColQwen3_5Wrapper,
    loader_kwargs=dict(
        torch_dtype=torch.bfloat16,
    ),
    name="athrael-soju/colqwen3.5-4.5B-v3",
    model_type=["late-interaction"],
    languages=["eng-Latn"],
    revision="4ad8f151e39bce3adcf88e0bdd72e724c7606638",
    release_date="2026-03-15",
    modalities=["image", "text"],
    n_parameters=4_600_000_000,
    n_embedding_parameters=635_699_200,
    memory_usage_mb=8660,
    max_tokens=262144,
    embed_dim=128,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "ColPali", "safetensors"],
    reference="https://huggingface.co/athrael-soju/colqwen3.5-4.5B-v3",
    similarity_fn_name=ScoringFunction.MAX_SIM,
    use_instructions=False,
    training_datasets=COLQWEN35_V3_TRAINING_DATA,
)



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class ESC50AudioReranking(AbsTaskRetrieval):
    """ESC-50 dataset adapted for audio reranking task.

    The Environmental Sound Classification 50 (ESC-50) dataset consists of 2000 environmental audio recordings
    suitable for benchmarking methods for environmental sound classification. This version is adapted for
    audio reranking where given a query audio from one of 50 environmental sound classes, the task is to
    rank positive audio samples (same class) higher than negative samples (different classes).

    Each query has 5 positive examples (same environmental sound class) and 16 negative examples
    (different environmental sound classes), creating a challenging audio-to-audio retrieval scenario
    with 21 total candidates per query. The dataset contains 200 queries providing robust evaluation
    across all 50 environmental sound categories.
    """

    metadata = TaskMetadata(
        name="ESC50AudioReranking",
        description="ESC-50 environmental sound dataset adapted for audio reranking. Given a query audio of environmental sounds, rank 5 relevant audio samples higher than 16 irrelevant ones from different sound classes. Contains 200 queries across 50 environmental sound categories for robust evaluation.",
        reference="https://github.com/karolpiczak/ESC-50",
        dataset={
            "path": "mteb/ESC50AudioReranking",
            "revision": "eff5c3112d71c4038377656f4d3ccb755fb7265f",
        },
        type="AudioReranking",
        category="a2a",
        modalities=["audio"],
        eval_splits=["test"],
        eval_langs=["zxx-Zxxx"],
        main_score="map_at_1000",
        date=("2015-01-01", "2015-12-31"),
        domains=["AudioScene"],
        task_subtypes=["Environment Sound Reranking"],
        license="cc-by-3.0",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{piczak2015dataset,
  author = {Piczak, Karol J.},
  booktitle = {Proceedings of the 23rd {Annual ACM Conference} on {Multimedia}},
  date = {2015-10-13},
  doi = {10.1145/2733373.2806390},
  isbn = {978-1-4503-3459-4},
  location = {{Brisbane, Australia}},
  pages = {1015--1018},
  publisher = {{ACM Press}},
  title = {{ESC}: {Dataset} for {Environmental Sound Classification}},
  url = {http://dl.acm.org/citation.cfm?doid=2733373.2806390},
}
""",
    )



FAQ = """### What do aggregate measures (Rank(Borda), Mean(Task), etc.) mean?

- **Rank(borda)** is computed based on the [borda count](https://en.wikipedia.org/wiki/Borda_count), where each task is treated as a preference voter, which gives votes on the models per their relative performance on the task. The best model obtains the highest number of votes. The model with the highest number of votes across tasks obtains the highest rank. The Borda rank tends to prefer models that perform well broadly across tasks. However, given that it is a rank it can be unclear if the two models perform similarly.
- **Mean(Task)**: This is a naïve average computed across all the tasks within the benchmark. This score is simple to understand and is continuous as opposed to the Borda rank. However, the mean can overvalue tasks with higher variance in its scores.
- **Mean(TaskType)**: This is a weighted average across different task categories, such as classification or retrieval. It is computed by first computing the average by task category and then computing the average on each category. Similar to the Mean(Task) this measure is continuous and tends to overvalue tasks with higher variance. This score also prefers models that perform well across all task categories.

### What does zero-shot mean?

A model is considered zero-shot if it is not trained on any splits of the datasets used to derive the tasks.
The percentages in the table indicate what portion of the benchmark can be considered out-of-distribution for a given model.
100% means the model has not been trained on any of the datasets in a given benchmark, and therefore the benchmark score can be interpreted as the model's overall generalization performance,
while 50% means the model has been finetuned on half of the tasks in the benchmark, thereby indicating that the benchmark results should be interpreted with a pinch of salt.
This definition creates a few edge cases. For instance, multiple models are typically trained on Wikipedia title and body pairs, but we do not define this as leakage on, e.g., “WikipediaRetrievalMultilingual” and “WikiClusteringP2P” as these datasets are not based on title-body pairs.
Distilled, further fine-tunes, or in other ways, derivative models inherit the datasets of their parent models.
Based on community feedback and research findings, this definition may change in the future. Please open a PR if you notice any mistakes or want to help us refine annotations, see [GitHub](https://github.com/embeddings-benchmark/mteb/blob/06489abca007261c7e6b11f36d4844c5ed5efdcb/mteb/models/bge_models.py#L91).

### What do the other columns mean?

- **Total Parameters**: This is the total number of parameters in the model including embedding parameters. A higher value means the model requires more CPU/GPU memory to run; thus, less is generally desirable.
- **Active Parameters**: This is the number of parameters actively used during inference (particularly relevant for MoE models). For dense models, this equals total parameters minus embedding parameters.
- **Embedding Dimension**: This is the vector dimension of the embeddings that the model produces. When saving embeddings to disk, a higher dimension will require more space, thus less is usually desirable.
- **Max tokens**: This refers to how many tokens (=word pieces) the model can process. Generally, a larger value is desirable.
- **Zero-shot**: This indicates if the model is zero-shot on the benchmark. For more information on zero-shot see the info box above.

### Why is a model missing or not showing up?

Possible reasons why a model may not show up in the leaderboard:

- **Filter Setting**: It is being filtered out with your current filter. By default, we do not show models that are not zero-shot on the benchmark.
You can change this setting in the model selection panel.
- **Missing Results**: The model may not have been run on the tasks in the benchmark. We only display models that have been run on at least one task
in the benchmark. For visualizations that require the mean across all tasks, we only display models that have been run on all tasks in the benchmark.
You can see existing results in the [results repository](https://github.com/embeddings-benchmark/results). This is also where new results are added via PR.
- **Missing Metadata**: Currently, we only show models for which we have metadata in [mteb](https://github.com/embeddings-benchmark/mteb).
You can follow this guide on how to add a [model](https://github.com/embeddings-benchmark/mteb/blob/main/docs/adding_a_model.md) and
see existing implementations [here](https://github.com/embeddings-benchmark/mteb/tree/main/mteb/models).
"""


ACKNOWLEDGEMENT = """
<div style="border-top: 1px solid #ddd; margin-top: 30px; padding-top: 10px; font-size: 0.85em; color: #666;">
  <p><strong>Acknowledgment:</strong> We thank <a href="https://cloud.google.com/">Google</a>, <a href="https://contextual.ai/">Contextual AI</a>, <a href="https://www.laude.org/">Laude Institute</a>, <a href="https://www.servicenow.com/">ServiceNow</a> and <a href="https://huggingface.co/">Hugging Face</a> for their generous sponsorship. If you'd like to sponsor us, please get in <a href="mailto:n.muennighoff@gmail.com">touch</a>.</p>
<div class="sponsor-image-about" style="display: flex; align-items: center; gap: 10px;">
    <a href="https://cloud.google.com/">
        <img src="https://img.icons8.com/?size=512&id=17949&format=png" width="60" height="55" style="padding: 10px;">
    </a>
    <a href="https://www.laude.org/">
        <img src="https://media.licdn.com/dms/image/v2/D4E0BAQEf_yGxYWCZPQ/company-logo_200_200/B4EZd7LecEHcAI-/0/1750118302768/laude_institute_logo?e=2147483647&v=beta&t=HqOMQxuGeHVjZsPIZ0vJXHpR3ZBH9OVo2aYK-f9ovio" width="60" height="55" style="padding: 10px;">
    </a>
    <a href="https://www.servicenow.com/">
        <img src="https://play-lh.googleusercontent.com/HdfHZ5jnfMM1Ep7XpPaVdFIVSRx82wKlRC_qmnHx9H1E4aWNp4WKoOcH0x95NAnuYg" width="60" height="55" style="padding: 10px;">
    </a>
    <a href="https://huggingface.co">
        <img src="https://raw.githubusercontent.com/embeddings-benchmark/mteb/main/docs/images/logos/hf_logo.png" width="60" height="55" style="padding: 10px;">
    </a>
</div>

  <p style="margin-top: 5px; font-size: 0.8em;">We also thank the following companies which provide API credits to evaluate their models: <a href="https://openai.com/">OpenAI</a>, <a href="https://www.voyageai.com/">Voyage AI</a></p>
</div>
"""



from mteb.abstasks.task_metadata import TaskMetadata
from mteb.abstasks.text.bitext_mining import AbsTaskBitextMining


class BornholmBitextMining(AbsTaskBitextMining):
    metadata = TaskMetadata(
        name="BornholmBitextMining",
        dataset={
            "path": "mteb/BornholmBitextMining",
            "revision": "5b02048bd75e79275aa91a1fce6cdfd3f4a391cb",
        },
        description="Danish Bornholmsk Parallel Corpus. Bornholmsk is a Danish dialect spoken on the island of Bornholm, Denmark. Historically it is a part of east Danish which was also spoken in Scania and Halland, Sweden.",
        reference="https://aclanthology.org/W19-6138/",
        type="BitextMining",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["dan-Latn"],
        main_score="f1",
        date=("2019-01-01", "2019-12-31"),
        domains=["Web", "Social", "Fiction", "Written"],
        license="cc-by-4.0",
        task_subtypes=["Dialect pairing"],
        annotations_creators="expert-annotated",
        dialect=["da-dan-bornholm"],
        sample_creation="created",
        bibtex_citation=r"""
@inproceedings{derczynskiBornholmskNaturalLanguage2019,
  author = {Derczynski, Leon and Kjeldsen, Alex Speed},
  booktitle = {Proceedings of the Nordic Conference of Computational Linguistics (2019)},
  date = {2019},
  file = {Available Version (via Google Scholar):/Users/au554730/Zotero/storage/FBQ73ZYN/Derczynski and Kjeldsen - 2019 - Bornholmsk natural language processing Resources .pdf:application/pdf},
  pages = {338--344},
  publisher = {Linköping University Electronic Press},
  shorttitle = {Bornholmsk natural language processing},
  title = {Bornholmsk natural language processing: Resources and tools},
  url = {https://pure.itu.dk/ws/files/84551091/W19_6138.pdf},
  urldate = {2024-04-24},
}
""",
        prompt="Retrieve parallel sentences.",
    )



from __future__ import annotations

import logging
from typing import TYPE_CHECKING

import numpy as np

from mteb._evaluators import SummarizationEvaluator
from mteb.abstasks._statistics_calculation import (
    calculate_score_statistics,
    calculate_text_statistics,
)
from mteb.abstasks.abstask import AbsTask
from mteb.models import EncoderProtocol
from mteb.types.statistics import (
    SplitDescriptiveStatistics,
)

if TYPE_CHECKING:
    from pathlib import Path

    from datasets import Dataset

    from mteb._evaluators.text.summarization_evaluator import SummarizationMetrics
    from mteb.models import MTEBModels
    from mteb.types import EncodeKwargs
    from mteb.types.statistics import (
        ScoreStatistics,
        TextStatistics,
    )

logger = logging.getLogger(__name__)


class SummarizationDescriptiveStatistics(SplitDescriptiveStatistics):
    """Descriptive statistics for Summarization

    Attributes:
        num_samples: number of samples in the dataset.
        number_of_characters: Total number of symbols in the dataset.

        text_statistics: Statistics for the text
        human_summaries_statistics: Statistics for human summaries
        machine_summaries_statistics: Statistics for machine summaries
        score_statistics: Statistics for the relevance scoresk
    """

    num_samples: int
    number_of_characters: int

    text_statistics: TextStatistics
    human_summaries_statistics: TextStatistics
    machine_summaries_statistics: TextStatistics
    score_statistics: ScoreStatistics


class AbsTaskSummarization(AbsTask):
    """Abstract class for summarization experiments.

    Attributes:
        dataset: HuggingFace dataset containing the data for the task. Should have columns:
            - text: The original text to be summarized.
            - human_summaries: A list of human-written summaries for the text.
            - machine_summaries: A list of machine-generated summaries for the text.
            - relevance: A list of relevance scores (integers) corresponding to each machine summary, indicating how relevant each summary is to the original text.
        min_score: Minimum possible relevance score (inclusive).
        max_score: Maximum possible relevance score (inclusive).
        human_summaries_column_name: Name of the column containing human summaries.
        machine_summaries_column_name: Name of the column containing machine summaries.
        text_column_name: Name of the column containing the original text.
        relevancy_column_name: Name of the column containing relevance scores.
        abstask_prompt: Prompt to use for the task for instruction model if not prompt is provided in TaskMetadata.prompt.
    """

    min_score: int
    max_score: int

    abstask_prompt = (
        "Given a news summary, retrieve other semantically similar summaries."
    )
    # SummEval has DeprecatedSummarizationEvaluator
    evaluator = SummarizationEvaluator
    text_column_name: str = "text"
    human_summaries_column_name: str = "human_summaries"
    machine_summaries_column_name: str = "machine_summaries"
    relevancy_column_name: str = "relevance"

    def _evaluate_subset(
        self,
        model: MTEBModels,
        data_split: Dataset,
        *,
        hf_split: str,
        hf_subset: str,
        encode_kwargs: EncodeKwargs,
        prediction_folder: Path | None = None,
        num_proc: int | None = None,
        **kwargs,
    ) -> SummarizationMetrics:
        if not isinstance(model, EncoderProtocol):
            raise TypeError("Expected model to be an instance of EncoderProtocol")

        normalized_scores = [
            (
                (np.array(x) - self.min_score) / (self.max_score - self.min_score)
            ).tolist()
            for x in data_split[self.relevancy_column_name]
        ]
        evaluator = self.evaluator(
            machine_summaries=data_split[self.machine_summaries_column_name],
            human_summaries=data_split[self.human_summaries_column_name],
            texts=data_split[self.text_column_name],
            gold_scores=normalized_scores,
            task_metadata=self.metadata,
            hf_split=hf_split,
            hf_subset=hf_subset,
            **kwargs,
        )
        scores = evaluator(model, encode_kwargs=encode_kwargs, num_proc=num_proc)
        if prediction_folder:
            self._save_task_predictions(
                scores,
                model,
                prediction_folder,
                hf_subset=hf_subset,
                hf_split=hf_split,
            )
        return evaluator._calculate_metrics(scores)

    def _calculate_descriptive_statistics_from_split(
        self, split: str, hf_subset: str | None = None, compute_overall: bool = False
    ) -> SummarizationDescriptiveStatistics:
        if hf_subset:
            text = self.dataset[hf_subset][split][self.text_column_name]
            human_summaries = self.dataset[hf_subset][split][
                self.human_summaries_column_name
            ]
            machine_summaries = self.dataset[hf_subset][split][
                self.machine_summaries_column_name
            ]
            relevance = self.dataset[hf_subset][split][self.relevancy_column_name]
        elif compute_overall:
            text = []
            human_summaries = []
            machine_summaries = []
            relevance = []

            for hf_subset in self.metadata.eval_langs:
                text.extend(self.dataset[hf_subset][split][self.text_column_name])
                human_summaries.extend(
                    self.dataset[hf_subset][split][self.human_summaries_column_name]
                )
                machine_summaries.extend(
                    self.dataset[hf_subset][split][self.machine_summaries_column_name]
                )
                relevance.extend(
                    self.dataset[hf_subset][split][self.relevancy_column_name]
                )
        else:
            text = self.dataset[split][self.text_column_name]
            human_summaries = self.dataset[split][self.human_summaries_column_name]
            machine_summaries = self.dataset[split][self.machine_summaries_column_name]
            relevance = self.dataset[split][self.relevancy_column_name]

        all_human_summaries = []
        for s in human_summaries:
            all_human_summaries.extend(s)

        all_machine_summaries = []
        for s in machine_summaries:
            all_machine_summaries.extend(s)

        text_statistics = calculate_text_statistics(text)
        human_summaries_statistics = calculate_text_statistics(all_human_summaries)
        machine_summaries_statistics = calculate_text_statistics(all_machine_summaries)

        relevance = [item for sublist in relevance for item in sublist]

        return SummarizationDescriptiveStatistics(
            num_samples=len(text),
            number_of_characters=(
                text_statistics["total_text_length"]
                + human_summaries_statistics["total_text_length"]
                + machine_summaries_statistics["total_text_length"]
            ),
            text_statistics=text_statistics,
            human_summaries_statistics=human_summaries_statistics,
            machine_summaries_statistics=machine_summaries_statistics,
            score_statistics=calculate_score_statistics(relevance),
        )



from mteb.abstasks.clustering import AbsTaskClustering
from mteb.abstasks.task_metadata import TaskMetadata


class LivedoorNewsClusteringv2(AbsTaskClustering):
    max_document_to_embed = 1107
    max_fraction_of_documents_to_embed = None

    metadata = TaskMetadata(
        name="LivedoorNewsClustering.v2",
        description="Clustering of the news reports of a Japanese news site, Livedoor News by RONDHUIT Co, Ltd. in 2012. It contains over 7,000 news report texts across 9 categories (topics). Version 2 updated on LivedoorNewsClustering by removing pairs where one of entries contain an empty sentences.",
        reference="https://github.com/sbintuitions/JMTEB",
        dataset={
            "path": "mteb/LivedoorNewsClustering.v2",
            "revision": "21637d8f5a8978f029d1175e7e70fd0e9d9ed51e",
        },
        type="Clustering",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["jpn-Jpan"],
        main_score="v_measure",
        date=("2000-01-01", "2014-02-09"),
        domains=["News", "Written"],
        task_subtypes=["Topic classification"],
        license="cc-by-nd-2.1-jp",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation="",
        adapted_from=["LivedoorNewsClustering"],
    )


class LivedoorNewsClustering(AbsTaskClustering):
    max_document_to_embed = 1107
    max_fraction_of_documents_to_embed = None

    metadata = TaskMetadata(
        name="LivedoorNewsClustering",
        description="Clustering of the news reports of a Japanese news site, Livedoor News by RONDHUIT Co, Ltd. in 2012. It contains over 7,000 news report texts across 9 categories (topics).",
        reference="https://github.com/sbintuitions/JMTEB",
        dataset={
            "path": "mteb/LivedoorNewsClustering",
            "revision": "4e1f50751b9dc2bbcbf56e3ebde55f82b74b6988",
        },
        type="Clustering",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["jpn-Jpan"],
        main_score="v_measure",
        date=("2000-01-01", "2014-02-09"),
        domains=["News", "Written"],
        task_subtypes=["Topic classification"],
        license="cc-by-nd-2.1-jp",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation="",
        superseded_by="LivedoorNewsClustering.v2",
    )



from mteb.models.model_meta import ModelMeta, ScoringFunction
from mteb.models.sentence_transformer_wrapper import SentenceTransformerEncoderWrapper

auto_g_embed_st = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    name="geoffsee/auto-g-embed-st",
    model_type=["dense"],
    revision="3e0bf6004ec386dea06d55dda4efe38fd96b5f7b",
    release_date="2026-02-08",
    languages=["eng-Latn"],
    open_weights=True,
    n_parameters=22_713_216,
    n_embedding_parameters=11_720_448,
    memory_usage_mb=87,
    embed_dim=384,
    license="mit",
    max_tokens=256,
    reference="https://huggingface.co/geoffsee/auto-g-embed-st",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=[
        "Sentence Transformers",
        "PyTorch",
        "safetensors",
        "Transformers",
    ],
    use_instructions=False,
    superseded_by=None,
    adapted_from="sentence-transformers/all-MiniLM-L6-v2",
    training_datasets=set(),  # no known overlap with MTEB datasets
    public_training_code="https://github.com/geoffsee/auto-g-embed",
    public_training_data=None,
    citation=None,
)



from .check_language_code import check_language_code
from .iso_mappings import ISO_TO_FAM, ISO_TO_FAM_LEVEL0, ISO_TO_LANGUAGE, ISO_TO_SCRIPT
from .language_scripts import LanguageScripts
from .programming_languages import PROGRAMMING_LANGS

__all__ = [
    "ISO_TO_FAM",
    "ISO_TO_FAM_LEVEL0",
    "ISO_TO_LANGUAGE",
    "ISO_TO_SCRIPT",
    "PROGRAMMING_LANGS",
    "LanguageScripts",
    "check_language_code",
]



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class IRPapersT2IRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="IRPapersT2IRetrieval",
        description="IRPAPERS is a collection of 166 Information Retrieval papers spanning 3,230 pages. Each page in the dataset is jointly represented as a base64 encoded string of the page image as well as an OCR-derived text transcription. IRPAPERS also contains 180 needle-in-the-haystack queries.",
        reference="https://arxiv.org/pdf/2602.17687",
        dataset={
            "path": "mteb/IRPapersRetrieval",
            "revision": "fc9ef6144f0df1b44782d6bd5b251f526dea8d54",
        },
        type="DocumentUnderstanding",
        category="t2i",
        eval_splits=["train"],
        eval_langs=["eng-Latn"],
        main_score="recall_at_5",
        date=("2026-01-01", "2026-01-01"),
        domains=["Academic"],
        task_subtypes=["Image Text Retrieval"],
        license="mit",
        annotations_creators="derived",
        dialect=[],
        modalities=["text", "image"],
        sample_creation="found",
        bibtex_citation=r"""
@misc{shorten2026,
  archiveprefix = {arXiv},
  author = {Connor Shorten and Augustas Skaburskas and Daniel M. Jones and Charles Pierse and Roberto Esposito and John Trengrove and Etienne Dilocker and Bob van Luijt},
  eprint = {2602.17687},
  primaryclass = {cs.IR},
  title = {IRPAPERS: A Visual Document Benchmark for Scientific Retrieval and Question Answering},
  url = {https://arxiv.org/pdf/2602.17687},
  year = {2026},
}
""",
    )

    def dataset_transform(self, num_proc: int | None = None, **kwargs) -> None:
        self.dataset["default"]["train"]["corpus"] = self.dataset["default"]["train"][
            "corpus"
        ].remove_columns("transcription")



from __future__ import annotations

import json
import logging
import warnings
from pathlib import Path
from typing import TYPE_CHECKING, Any

import numpy as np

from ._hash_utils import _hash_item

if TYPE_CHECKING:
    from mteb.types import Array

logger = logging.getLogger(__name__)


class NumpyCache:
    """Generic vector cache for both text and images."""

    def __init__(self, directory: str | Path, initial_vectors: int = 100_000):
        self.directory = Path(directory)
        self.directory.mkdir(parents=True, exist_ok=True)
        self.vectors_file = self.directory / "vectors.npy"
        self.index_file = self.directory / "index.json"
        self.dimension_file = self.directory / "dimension"
        self.hash_to_index: dict[str, int] = {}
        self.vectors: np.memmap | None = None
        self.vector_dim: int | None = None
        self.initial_vectors = initial_vectors
        logger.info(f"Initialized VectorCacheMap in directory: {self.directory}")
        self._initialize_vectors_file()

    def add(self, items: list[dict[str, Any]], vectors: Array) -> None:
        """Add a vector to the cache."""
        try:
            if self.vector_dim is None:
                self.vector_dim = (
                    vectors.shape[0] if vectors.ndim == 1 else vectors.shape[1]
                )
                self._initialize_vectors_file()
                self._save_dimension()
                logger.info(f"Initialized vector dimension to {self.vector_dim}")

            if self.vectors is None:
                raise RuntimeError(
                    "Vectors file not initialized. Call _initialize_vectors_file() first."
                )

            for item, vec in zip(items, vectors):
                item_hash = _hash_item(item)
                if item_hash in self.hash_to_index:
                    msg = f"Hash collision or duplicate item for hash {item_hash}. Overwriting existing vector."
                    logger.warning(msg)
                    warnings.warn(msg)
                    index = self.hash_to_index[item_hash]
                else:
                    index = len(self.hash_to_index)
                    if index >= len(self.vectors):
                        self._double_vectors_file()
                    self.hash_to_index[item_hash] = index

                self.vectors[index] = vec
                logger.debug(
                    f"Added new item-vector pair. Total pairs: {len(self.hash_to_index)}"
                )
        except Exception as e:
            logger.error(f"Error adding item-vector pair: {str(e)}")
            raise

    def _initialize_vectors_file(self) -> None:
        if self.vector_dim is None:
            logger.info("Vector dimension not set. Waiting for first add() call.")
            return
        if not self.vectors_file.exists():
            logger.info(
                f"Creating initial vectors file with {self.initial_vectors} vectors"
            )
            self.vectors = np.memmap(
                self.vectors_file,
                dtype="float32",
                mode="w+",
                shape=(self.initial_vectors, self.vector_dim),
            )
        else:
            self.vectors = np.memmap(
                self.vectors_file,
                dtype="float32",
                mode="r+",
                shape=(-1, self.vector_dim),
            )
        logger.info(f"Vectors file initialized with shape: {self.vectors.shape}")

    def _double_vectors_file(self) -> None:
        if self.vectors is None or self.vector_dim is None:
            raise RuntimeError(
                "Vectors file not initialized. Call _initialize_vectors_file() first."
            )
        current_size = len(self.vectors)
        new_size = current_size * 2
        logger.info(f"Doubling vectors file from {current_size} to {new_size} vectors")
        self.vectors.flush()
        new_vectors = np.memmap(
            str(self.vectors_file),
            dtype=np.float32,
            mode="r+",
            shape=(new_size, self.vector_dim),
        )
        new_vectors[:current_size] = self.vectors[:]
        self.vectors = new_vectors

    def _save_dimension(self) -> None:
        with self.dimension_file.open("w") as f:
            f.write(str(self.vector_dim))
        logger.info(
            f"Saved vector dimension {self.vector_dim} to {self.dimension_file}"
        )

    def _load_dimension(self) -> None:
        if self.dimension_file.exists():
            with self.dimension_file.open() as f:
                self.vector_dim = int(f.read().strip())
            logger.info(
                f"Loaded vector dimension {self.vector_dim} from {self.dimension_file}"
            )
        else:
            msg = "Dimension file not found. Vector dimension remains uninitialized."
            logger.warning(msg)
            warnings.warn(msg)

    def save(self) -> None:
        """Persist VectorCacheMap to disk."""
        try:
            if self.vectors is not None:
                self.vectors.flush()

            # Convert hash_to_index dict to a format suitable for JSON
            # JSON doesn't support integer keys, so we keep everything as strings
            serializable_index = {
                str(hash_): int(index)  # Ensure indices are serialized as integers
                for hash_, index in self.hash_to_index.items()
            }

            with self.index_file.open("w", encoding="utf-8") as f:
                json.dump(serializable_index, f, indent=2)
            self._save_dimension()
            logger.info(f"Saved VectorCacheMap to {self.directory}")
        except Exception as e:
            logger.error(f"Error saving VectorCacheMap: {str(e)}")
            raise

    def load(self) -> None:
        """Load VectorCacheMap from disk."""
        try:
            self._load_dimension()
            if self.index_file.exists() and self.vectors_file.exists():
                with self.index_file.open(encoding="utf-8") as f:
                    loaded_index = json.load(f)
                    self.hash_to_index = {
                        str(hash_): int(index)  # Ensure we maintain the correct types
                        for hash_, index in loaded_index.items()
                    }

                if self.vector_dim is not None:
                    self.vectors = np.memmap(
                        self.vectors_file,
                        dtype="float32",
                        mode="r+",
                        shape=(-1, self.vector_dim),
                    )
                    logger.info(f"Loaded vectors file with shape: {self.vectors.shape}")
                else:
                    msg = "Vector dimension not set. Unable to load vectors file."
                    logger.warning(msg)
                    warnings.warn(msg)
                logger.info(f"Loaded VectorCacheMap from {self.directory}")
            else:
                msg = "No existing files found. Initialized empty VectorCacheMap."
                logger.warning(msg)
                warnings.warn(msg)
        except Exception as e:
            logger.error(f"Error loading VectorCacheMap: {str(e)}")
            raise

    def get_vector(self, item: dict[str, Any]) -> Array | None:
        """Retrieve vector from index by hash."""
        if self.vectors is None:
            return None

        try:
            item_hash = _hash_item(item)
            if item_hash not in self.hash_to_index:
                logger.debug(f"Item hash not found in index: {item_hash}")
                return None
            index = self.hash_to_index[item_hash]
            return self.vectors[index]
        except Exception as e:
            logger.error(f"Error retrieving vector for item: {str(e)}")
            raise

    def __contains__(self, item: dict[str, Any]) -> bool:
        return _hash_item(item) in self.hash_to_index

    def __del__(self):
        self.close()

    def close(self) -> None:
        """Delete all ve"""
        if hasattr(self, "vectors") and self.vectors is not None:
            self.vectors.flush()
            del self.vectors
            self.vectors = None



import datasets
from sklearn.linear_model import LogisticRegression
from sklearn.multioutput import MultiOutputClassifier

from mteb.abstasks import AbsTaskMultilabelClassification
from mteb.abstasks.task_metadata import TaskMetadata


class FSD2019KaggleMultilingualClassification(AbsTaskMultilabelClassification):
    metadata = TaskMetadata(
        name="FSD2019Kaggle",
        description="Multilabel Audio Classification.",
        reference="https://huggingface.co/datasets/confit/fsdkaggle2019-parquet",  # "https://huggingface.co/datasets/CLAPv2/FSD50K",
        dataset={
            "path": "mteb/fsdkaggle2019-parquet",
            "revision": "ff7e6d9e57951b90024fcb6d4aef95d97f4fc64e",
        },  # this is actually used to download the data
        type="AudioMultilabelClassification",
        category="a2t",
        eval_splits=["test"],
        eval_langs={"curated": ["eng-Latn"], "noisy": ["eng-Latn"]},
        main_score="accuracy",
        date=(
            "2020-01-01",
            "2020-01-30",
        ),  # Estimated date when this dataset was committed, what should be the second tuple?
        domains=["Web"],  # obtained from Freesound - online collaborative platform
        task_subtypes=["Environment Sound Classification"],
        license="cc-by-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        modalities=["audio"],
        sample_creation="found",
        bibtex_citation=r"""
@dataset{eduardo_fonseca_2020_3612637,
  author = {Eduardo Fonseca and
Manoj Plakal and
Frederic Font and
Daniel P. W. Ellis and
Xavier Serra},
  doi = {10.5281/zenodo.3612637},
  month = jan,
  publisher = {Zenodo},
  title = {FSDKaggle2019},
  url = {https://doi.org/10.5281/zenodo.3612637},
  version = {1.0},
  year = {2020},
}
""",
    )

    evaluator_model = MultiOutputClassifier(estimator=LogisticRegression())
    input_column_name: str = "audio"
    label_column_name: str = "sound"
    samples_per_label: int = 8

    def load_data(self, **kwargs):
        """Load dataset from HuggingFace hub and convert it to the standard format."""
        if self.data_loaded:
            return

        self.dataset = {}
        for lang in self.hf_subsets:
            self.dataset[lang] = datasets.load_dataset(
                name=lang, **self.metadata.dataset
            )

        self.dataset_transform()
        self.data_loaded = True



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class JapaneseLegal1Retrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="JapaneseLegal1Retrieval",
        description="Japanese legal regulation retrieval dataset with queries about government regulations, ministry ordinances, and administrative law.",
        reference="https://huggingface.co/datasets/mteb-private/JapaneseLegal1Retrieval-sample",
        dataset={
            "path": "mteb-private/JapaneseLegal1Retrieval",
            "revision": "d653557fe66bb6af2b0e2adfc371a24554cf11ce",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["jpn-Jpan"],
        main_score="ndcg_at_10",
        date=("2024-01-01", "2024-01-01"),
        domains=["Legal", "Written"],
        task_subtypes=["Article retrieval"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation="",
        is_public=False,
        contributed_by="Voyage AI",
    )



from datasets import load_dataset

from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata

_EVAL_SPLIT = "test"

_LANGUAGES = [
    "acm_Arab",
    "afr_Latn",
    "als_Latn",
    "amh_Ethi",
    "apc_Arab",
    "arb_Arab",
    "arb_Latn",
    "ars_Arab",
    "ary_Arab",
    "arz_Arab",
    "asm_Beng",
    "azj_Latn",
    "bam_Latn",
    "ben_Beng",
    "ben_Latn",
    "bod_Tibt",
    "bul_Cyrl",
    "cat_Latn",
    "ceb_Latn",
    "ces_Latn",
    "ckb_Arab",
    "dan_Latn",
    "deu_Latn",
    "ell_Grek",
    "eng_Latn",
    "est_Latn",
    "eus_Latn",
    "fin_Latn",
    "fra_Latn",
    "fuv_Latn",
    "gaz_Latn",
    "grn_Latn",
    "guj_Gujr",
    "hat_Latn",
    "hau_Latn",
    "heb_Hebr",
    "hin_Deva",
    "hin_Latn",
    "hrv_Latn",
    "hun_Latn",
    "hye_Armn",
    "ibo_Latn",
    "ilo_Latn",
    "ind_Latn",
    "isl_Latn",
    "ita_Latn",
    "jav_Latn",
    "jpn_Jpan",
    "kac_Latn",
    "kan_Knda",
    "kat_Geor",
    "kaz_Cyrl",
    "kea_Latn",
    "khk_Cyrl",
    "khm_Khmr",
    "kin_Latn",
    "kir_Cyrl",
    "kor_Hang",
    "lao_Laoo",
    "lin_Latn",
    "lit_Latn",
    "lug_Latn",
    "luo_Latn",
    "lvs_Latn",
    "mal_Mlym",
    "mar_Deva",
    "mkd_Cyrl",
    "mlt_Latn",
    "mri_Latn",
    "mya_Mymr",
    "nld_Latn",
    "nob_Latn",
    "npi_Deva",
    "npi_Latn",
    "nso_Latn",
    "nya_Latn",
    "ory_Orya",
    "pan_Guru",
    "pbt_Arab",
    "pes_Arab",
    "plt_Latn",
    "pol_Latn",
    "por_Latn",
    "ron_Latn",
    "rus_Cyrl",
    "shn_Mymr",
    "sin_Latn",
    "sin_Sinh",
    "slk_Latn",
    "slv_Latn",
    "sna_Latn",
    "snd_Arab",
    "som_Latn",
    "sot_Latn",
    "spa_Latn",
    "srp_Cyrl",
    "ssw_Latn",
    "sun_Latn",
    "swe_Latn",
    "swh_Latn",
    "tam_Taml",
    "tel_Telu",
    "tgk_Cyrl",
    "tgl_Latn",
    "tha_Thai",
    "tir_Ethi",
    "tsn_Latn",
    "tso_Latn",
    "tur_Latn",
    "ukr_Cyrl",
    "urd_Arab",
    "urd_Latn",
    "uzn_Latn",
    "vie_Latn",
    "war_Latn",
    "wol_Latn",
    "xho_Latn",
    "yor_Latn",
    "zho_Hans",
    "zho_Hant",
    "zsm_Latn",
    "zul_Latn",
]


def get_lang_pairs() -> dict[str, list[str]]:
    # add pairs with same language as the source and target
    # add pairs with english as source or target
    lang_pairs = {}
    for x in _LANGUAGES:
        pair = f"{x}-{x}"
        lang_pairs[pair] = [x.replace("_", "-"), x.replace("_", "-")]

        if x != "eng_Latn":
            pair = f"{x}-eng_Latn"
            lang_pairs[pair] = [x.replace("_", "-"), "eng-Latn"]
            pair = f"eng_Latn-{x}"
            lang_pairs[pair] = ["eng-Latn", x.replace("_", "-")]

    # add pairs for languages with a base script and a Latn script
    lang_base_scripts = [
        "arb_Arab",
        "ben_Beng",
        "hin_Deva",
        "npi_Deva",
        "sin_Sinh",
        "urd_Arab",
    ]
    for lang_base_script in lang_base_scripts:
        lang = lang_base_script.split("_")[0]
        lang_latn_script = f"{lang}_Latn"
        pair = f"{lang_base_script}-{lang_latn_script}"
        lang_pairs[pair] = [
            lang_base_script.replace("_", "-"),
            lang_latn_script.replace("_", "-"),
        ]
        pair = f"{lang_latn_script}-{lang_base_script}"
        lang_pairs[pair] = [
            lang_latn_script.replace("_", "-"),
            lang_base_script.replace("_", "-"),
        ]

    return lang_pairs


_LANGUAGES_MAPPING = get_lang_pairs()


class BelebeleRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="BelebeleRetrieval",
        dataset={
            "path": "mteb/belebele",
            "revision": "979a211276faa22f671e69d096634193567cfd05",
        },
        description=(
            "Belebele is a multiple-choice machine reading comprehension (MRC) dataset spanning 122 language variants "
            + "(including 115 distinct languages and their scripts)"
        ),
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=[_EVAL_SPLIT],
        eval_langs=_LANGUAGES_MAPPING,
        reference="https://arxiv.org/abs/2308.16884",
        main_score="ndcg_at_10",
        license="cc-by-sa-4.0",
        domains=["Web", "News", "Written"],
        sample_creation="created",  # number of languages * 900
        date=("2023-08-31", "2023-08-31"),
        task_subtypes=["Question answering"],
        annotations_creators="expert-annotated",
        dialect=[],
        bibtex_citation=r"""
@article{bandarkar2023belebele,
  author = {Lucas Bandarkar and Davis Liang and Benjamin Muller and Mikel Artetxe and Satya Narayan Shukla and Donald Husa and Naman Goyal and Abhinandan Krishnan and Luke Zettlemoyer and Madian Khabsa},
  journal = {arXiv preprint arXiv:2308.16884},
  title = {The Belebele Benchmark: a Parallel Reading Comprehension Dataset in 122 Language Variants},
  year = {2023},
}
""",
    )

    def load_data(self, **kwargs) -> None:
        if self.data_loaded:
            return

        self.dataset = load_dataset(**self.metadata.dataset)

        self.queries = {lang_pair: {_EVAL_SPLIT: {}} for lang_pair in self.hf_subsets}
        self.corpus = {lang_pair: {_EVAL_SPLIT: {}} for lang_pair in self.hf_subsets}
        self.relevant_docs = {
            lang_pair: {_EVAL_SPLIT: {}} for lang_pair in self.hf_subsets
        }

        for lang_pair in self.hf_subsets:
            languages = self.metadata.eval_langs[lang_pair]
            lang_corpus, lang_question = (
                languages[0].replace("-", "_"),
                languages[1].replace("-", "_"),
            )
            ds_corpus = self.dataset[lang_corpus]
            ds_question = self.dataset[lang_question]

            question_ids = {}
            for row in ds_question:
                question = row["question"]
                if question not in question_ids:
                    question_ids[question] = len(question_ids)

            link_to_context_id = {}
            context_idx = 0
            for row in ds_corpus:
                if row["link"] not in link_to_context_id:
                    context_id = f"C{context_idx}"
                    link_to_context_id[row["link"]] = context_id
                    self.corpus[lang_pair][_EVAL_SPLIT][context_id] = {
                        "title": "",
                        "text": row["flores_passage"],
                    }
                    context_idx = context_idx + 1

            for row in ds_question:
                query = row["question"]
                query_id = f"Q{question_ids[query]}"
                self.queries[lang_pair][_EVAL_SPLIT][query_id] = query

                context_link = row["link"]
                context_id = link_to_context_id[context_link]
                if query_id not in self.relevant_docs[lang_pair][_EVAL_SPLIT]:
                    self.relevant_docs[lang_pair][_EVAL_SPLIT][query_id] = {}
                self.relevant_docs[lang_pair][_EVAL_SPLIT][query_id][context_id] = 1

        self.data_loaded = True



from mteb.models.model_meta import ModelMeta
from mteb.models.sentence_transformer_wrapper import (
    SentenceTransformerEncoderWrapper,
)

# Define task instructions with specific task names
task_instructions = {
    "Classification": "Generate a representation for this text that can be used for classification:",
    "Clustering": "Generate a representation for this text that can be used for clustering:",
    "Retrieval": "Generate a representation for this text that can be used for retrieval:",
    "STS": "Generate a representation for this text that can be used for semantic similarity:",
    "PairClassification": "Generate a representation for this text pair that can be used for classification:",
    "Reranking": "Generate a representation for this text that can be used for reranking:",
    "Summarization": "Generate a representation for this text that can be used for summarization:",
}

searchmap_preview = ModelMeta(
    loader=SentenceTransformerEncoderWrapper,
    loader_kwargs={
        "model_prompts": task_instructions,
    },
    name="VPLabs/SearchMap_Preview",
    model_type=["dense"],
    revision="69de17ef48278ed08ba1a4e65ead8179912b696e",
    languages=["eng-Latn"],
    open_weights=True,
    use_instructions=True,
    release_date="2025-03-05",
    n_parameters=435_000_000,
    n_embedding_parameters=31260672,
    memory_usage_mb=1660,
    embed_dim=4096,
    license="mit",
    max_tokens=8192,
    reference="https://huggingface.co/VPLabs/SearchMap_Preview",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "Transformers", "safetensors"],
    public_training_code=None,
    public_training_data=None,
    training_datasets=None,
    adapted_from="NovaSearch/stella_en_400M_v5",
    citation="""@misc{vectorpath2025searchmap,
  title={SearchMap: Conversational E-commerce Search Embedding Model},
  author={VectorPath Research Team},
  year={2025},
  publisher={Hugging Face},
  journal={HuggingFace Model Hub},
}
""",
)



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata

_LANGUAGES = {
    "fas": ["fas-Arab"],
    "rus": ["rus-Cyrl"],
    "zho": ["zho-Hans"],
}


class NeuCLIR2022Retrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="NeuCLIR2022Retrieval",
        description="The task involves identifying and retrieving the documents that are relevant to the queries.",
        reference="https://neuclir.github.io/",
        dataset={
            "path": "mteb/NeuCLIR2022Retrieval",
            "revision": "95cad8671c1c31908766a689755c307f4770411f",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=_LANGUAGES,
        main_score="ndcg_at_20",
        date=("2021-08-01", "2022-06-30"),
        domains=["News", "Written"],
        task_subtypes=[],
        license="odc-by",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@article{lawrie2023overview,
  author = {Lawrie, Dawn and MacAvaney, Sean and Mayfield, James and McNamee, Paul and Oard, Douglas W and Soldaini, Luca and Yang, Eugene},
  journal = {arXiv preprint arXiv:2304.12367},
  title = {Overview of the TREC 2022 NeuCLIR track},
  year = {2023},
}
""",
    )


class NeuCLIR2022RetrievalHardNegatives(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="NeuCLIR2022RetrievalHardNegatives",
        description="The task involves identifying and retrieving the documents that are relevant to the queries. The hard negative version has been created by pooling the 250 top documents per query from BM25, e5-multilingual-large and e5-mistral-instruct.",
        reference="https://neuclir.github.io/",
        dataset={
            "path": "mteb/NeuCLIR2022RetrievalHardNegatives",
            "revision": "cc5a0360c8bdf1e4afa19d2ea8ec111c0ca96335",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=_LANGUAGES,
        main_score="ndcg_at_20",
        date=("2021-08-01", "2022-06-30"),
        domains=["News", "Written"],
        task_subtypes=[],
        license="odc-by",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@article{lawrie2023overview,
  author = {Lawrie, Dawn and MacAvaney, Sean and Mayfield, James and McNamee, Paul and Oard, Douglas W and Soldaini, Luca and Yang, Eugene},
  journal = {arXiv preprint arXiv:2304.12367},
  title = {Overview of the TREC 2022 NeuCLIR track},
  year = {2023},
}
""",
        adapted_from=["NeuCLIR2022Retrieval"],
    )



from mteb.abstasks.aggregate_task_metadata import AggregateTaskMetadata
from mteb.abstasks.aggregated_task import AbsTaskAggregate
from mteb.tasks.classification import (
    SynPerChatbotConvSAAnger,
    SynPerChatbotConvSAFear,
    SynPerChatbotConvSAFriendship,
    SynPerChatbotConvSAHappiness,
    SynPerChatbotConvSAJealousy,
    SynPerChatbotConvSALove,
    SynPerChatbotConvSASadness,
    SynPerChatbotConvSASatisfaction,
    SynPerChatbotConvSASurprise,
)

task_list_cqa = [
    SynPerChatbotConvSAAnger(),
    SynPerChatbotConvSASatisfaction(),
    SynPerChatbotConvSAFriendship(),
    SynPerChatbotConvSAFear(),
    SynPerChatbotConvSAJealousy(),
    SynPerChatbotConvSASurprise(),
    SynPerChatbotConvSALove(),
    SynPerChatbotConvSASadness(),
    SynPerChatbotConvSAHappiness(),
]


class SynPerChatbotConvSAClassification(AbsTaskAggregate):
    metadata = AggregateTaskMetadata(
        name="SynPerChatbotConvSAClassification",
        description="SynPerChatbotConvSAClassification",
        reference="",
        tasks=task_list_cqa,
        main_score="accuracy",
        type="Classification",
        eval_splits=["test"],
        bibtex_citation=""" """,
    )



from __future__ import annotations

import logging
from typing import TYPE_CHECKING, Any

import torch
from tqdm.auto import tqdm
from transformers import AutoProcessor, EncodecModel

from mteb._create_dataloaders import AudioCollator
from mteb._requires_package import requires_audio_dependencies
from mteb.models import ModelMeta
from mteb.models.abs_encoder import AbsEncoder

if TYPE_CHECKING:
    from torch.utils.data import DataLoader

    from mteb import TaskMetadata
    from mteb.types import Array, BatchedInput, PromptType
    from mteb.types._encoder_io import AudioInput


logger = logging.getLogger(__name__)


class EncodecWrapper(AbsEncoder):
    def __init__(
        self,
        model_name: str,
        revision: str,
        device: str = "cuda" if torch.cuda.is_available() else "cpu",
        max_audio_length_seconds: float = 30.0,
        **kwargs: Any,
    ):
        requires_audio_dependencies()
        self.model_name = model_name
        self.device = device
        self.max_audio_length_seconds = max_audio_length_seconds

        self.model = EncodecModel.from_pretrained(model_name, revision=revision).to(
            device
        )
        self.model.eval()

        self.processor = AutoProcessor.from_pretrained(model_name)
        self.sampling_rate = self.processor.sampling_rate  # 24000 Hz typically

    def get_audio_embeddings(
        self,
        inputs: DataLoader[AudioInput],
        show_progress_bar: bool = True,
        **kwargs: Any,
    ) -> Array:
        max_samples = int(self.max_audio_length_seconds * self.sampling_rate)
        inputs.collate_fn = AudioCollator(self.sampling_rate, max_samples)

        all_embeddings = []

        for batch in tqdm(
            inputs,
            disable=not show_progress_bar,
        ):
            audio_array = [audio["array"] for audio in batch["audio"]]
            audio_arrays = []
            for array in audio_array:
                # Ensure minimum length for encoder (Encodec needs ~320 samples per frame)
                # Use 1 second minimum to be safe
                min_samples = self.sampling_rate
                if array.shape[-1] < min_samples:
                    padding = torch.zeros(min_samples - array.shape[-1])
                    array = torch.cat([array, padding])

                audio_arrays.append(array.numpy())

            with torch.no_grad():
                # Use processor for batch padding (truncation/min-length done manually above)
                processed = self.processor(
                    raw_audio=audio_arrays,
                    sampling_rate=self.sampling_rate,
                    padding=True,
                    return_tensors="pt",
                )
                input_values = processed["input_values"].to(self.device)

                # Add channel dimension if needed (B, T) -> (B, 1, T)
                if input_values.dim() == 2:
                    input_values = input_values.unsqueeze(1)

                # Get the latent representations directly from the encoder
                latent = self.model.encoder(input_values)

                # Validate latent has time frames
                if latent.shape[2] == 0:
                    raise ValueError(
                        f"Encodec encoder produced 0 time frames. "
                        f"Input shape: {input_values.shape}, latent shape: {latent.shape}"
                    )

                # Apply mean pooling over the time dimension to get fixed-size embeddings
                embeddings = torch.mean(latent, dim=2)  # Average over time dimension

                # Normalize embeddings
                embeddings = embeddings / embeddings.norm(dim=-1, keepdim=True)

                all_embeddings.append(embeddings.cpu().detach())

        return torch.cat(all_embeddings, dim=0).numpy()

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        if "audio" not in inputs.dataset.features:
            raise ValueError("EncodecWrapper only supports audio inputs.")
        return self.get_audio_embeddings(inputs, **kwargs)


encodec_24khz = ModelMeta(
    loader=EncodecWrapper,
    name="facebook/encodec_24khz",
    languages=["eng-Latn"],
    open_weights=True,
    revision="c1dbe2ae3f1de713481a3b3e7c47f357092ee040",
    release_date="2022-10-25",
    max_tokens=None,
    n_parameters=23_273_218,
    n_embedding_parameters=0,
    memory_usage_mb=88,
    embed_dim=128,
    license="cc-by-nc-4.0",
    reference="https://huggingface.co/facebook/encodec_24khz",
    similarity_fn_name="cosine",
    framework=["PyTorch"],
    use_instructions=False,
    public_training_code="https://github.com/facebookresearch/encodec",
    public_training_data=None,
    training_datasets=None,  # ["AudioSet", "VCTK", "DNS-Challenge"],
    modalities=["audio"],
    citation="""
@misc{défossez2022highfidelityneuralaudio,
      title={High Fidelity Neural Audio Compression},
      author={Alexandre Défossez and Jade Copet and Gabriel Synnaeve and Yossi Adi},
      year={2022},
      eprint={2210.13438},
      archivePrefix={arXiv},
      primaryClass={eess.AS},
      url={https://arxiv.org/abs/2210.13438},
}""",
)



from __future__ import annotations

from pathlib import Path
from typing import TYPE_CHECKING, Any

import torch
from tqdm.auto import tqdm

from mteb._requires_package import requires_image_dependencies
from mteb.models.abs_encoder import AbsEncoder
from mteb.models.model_meta import ModelMeta, ScoringFunction

if TYPE_CHECKING:
    from torch.utils.data import DataLoader

    from mteb.abstasks.task_metadata import TaskMetadata
    from mteb.types import Array, BatchedInput, PromptType

EVA_CLIP_CITATION = """@article{EVA-CLIP,
      title={EVA-CLIP: Improved Training Techniques for CLIP at Scale},
      author={Sun, Quan and Fang, Yuxin and Wu, Ledell and Wang, Xinlong and Cao, Yue},
      journal={arXiv preprint arXiv:2303.15389},
      year={2023}
}"""


def evaclip_loader(model_name, **kwargs):
    try:
        import sys

        sys.path.insert(0, str(Path.cwd() / "EVA" / "EVA-CLIP" / "rei"))

        from eva_clip import create_model_and_transforms, get_tokenizer
    except ImportError:
        # https://github.com/baaivision/EVA/tree/master/EVA-CLIP#setup
        raise ImportError(
            "Please run `git clone git@github.com:baaivision/EVA.git`,"
            "`pip install ninja timm`"
            "`pip install -v -U git+https://github.com/facebookresearch/xformers.git@main#egg=xformers`"
            "`git clone https://github.com/NVIDIA/apex && cd apex && pip install -v --disable-pip-version-check --no-build-isolation --no-cache-dir ./`"
        )

    class EvaCLIPWrapper(AbsEncoder):
        def __init__(
            self,
            model_name: str,
            device: str = "cuda" if torch.cuda.is_available() else "cpu",
            **kwargs: Any,
        ):
            requires_image_dependencies()

            self.model_name = model_name
            self.device = device
            pretrained = "eva_clip"  # or "/path/to/EVA02_CLIP_B_psz16_s8B.pt"
            self.model, _, self.img_preprocess = create_model_and_transforms(
                model_name, pretrained, force_custom_clip=True, device=device
            )
            self.model.eval()
            self.tokenizer = get_tokenizer(model_name)

        def get_text_embeddings(
            self,
            texts: DataLoader[BatchedInput],
            show_progress_bar: bool = True,
            **kwargs: Any,
        ):
            all_text_embeddings = []

            with torch.no_grad(), torch.cuda.amp.autocast():
                for batch in tqdm(
                    texts, disable=not show_progress_bar, desc="Text Encoding"
                ):
                    inputs = self.tokenizer(batch["text"])
                    text_outputs = self.model.encode_text(inputs.to(self.device))
                    all_text_embeddings.append(text_outputs.cpu())

            all_text_embeddings = torch.cat(all_text_embeddings, dim=0)
            return all_text_embeddings

        def get_image_embeddings(
            self,
            images: DataLoader[BatchedInput],
            show_progress_bar: bool = True,
            **kwargs: Any,
        ):
            all_image_embeddings = []

            with torch.no_grad(), torch.cuda.amp.autocast():
                for batch in tqdm(
                    images, disable=not show_progress_bar, desc="Image Encoding"
                ):
                    inputs = torch.vstack(
                        [self.img_preprocess(b).unsqueeze(0) for b in batch["image"]]
                    )
                    image_outputs = self.model.encode_image(inputs.to(self.device))
                    all_image_embeddings.append(image_outputs.cpu())

            all_image_embeddings = torch.cat(all_image_embeddings, dim=0)
            return all_image_embeddings

        def encode(
            self,
            inputs: DataLoader[BatchedInput],
            *,
            task_metadata: TaskMetadata,
            hf_split: str,
            hf_subset: str,
            prompt_type: PromptType | None = None,
            **kwargs: Any,
        ) -> Array:
            text_embeddings = None
            image_embeddings = None
            if "text" in inputs.dataset.features:
                text_embeddings = self.get_text_embeddings(inputs, **kwargs)
            if "image" in inputs.dataset.features:
                image_embeddings = self.get_image_embeddings(inputs, **kwargs)

            if text_embeddings is not None and image_embeddings is not None:
                if len(text_embeddings) != len(image_embeddings):
                    raise ValueError(
                        "The number of texts and images must have the same length"
                    )
                fused_embeddings = text_embeddings + image_embeddings
                return fused_embeddings
            elif text_embeddings is not None:
                return text_embeddings
            elif image_embeddings is not None:
                return image_embeddings
            raise ValueError

    return EvaCLIPWrapper(model_name, **kwargs)


training_code = "https://github.com/baaivision/EVA/tree/master/EVA-CLIP"
training_datasets = set(
    # COYO-700M, random sample 400M. https://github.com/kakaobrain/coyo-dataset
    # LAION-2B, random sample 1.6B. https://laion.ai/blog/laion-5b/
)
laion_2b = set(
    # LAION-2B
)

EVA02_CLIP_B_16 = ModelMeta(
    loader=evaclip_loader,
    name="QuanSun/EVA02-CLIP-B-16",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="11afd202f2ae80869d6cef18b1ec775e79bd8d12",
    release_date="2023-04-26",
    modalities=["image", "text"],
    n_parameters=149_000_000,
    n_embedding_parameters=None,
    memory_usage_mb=568,
    max_tokens=77,
    embed_dim=512,
    license="mit",
    open_weights=True,
    public_training_code=training_code,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/QuanSun/EVA-CLIP",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=training_datasets,
    citation=EVA_CLIP_CITATION,
)

EVA02_CLIP_L_14 = ModelMeta(
    loader=evaclip_loader,
    name="QuanSun/EVA02-CLIP-L-14",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="11afd202f2ae80869d6cef18b1ec775e79bd8d12",
    release_date="2023-04-26",
    modalities=["image", "text"],
    n_parameters=428_000_000,
    n_embedding_parameters=None,
    memory_usage_mb=1633,
    max_tokens=77,
    embed_dim=768,
    license="mit",
    open_weights=True,
    public_training_code=training_code,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/QuanSun/EVA-CLIP",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=training_datasets,
    citation=EVA_CLIP_CITATION,
)

EVA02_CLIP_bigE_14 = ModelMeta(
    loader=evaclip_loader,
    name="QuanSun/EVA02-CLIP-bigE-14",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="11afd202f2ae80869d6cef18b1ec775e79bd8d12",
    release_date="2023-04-26",
    modalities=["image", "text"],
    n_parameters=4_700_000_000,
    n_embedding_parameters=None,
    memory_usage_mb=17929,
    max_tokens=77,
    embed_dim=1024,
    license="mit",
    open_weights=True,
    public_training_code=training_code,
    public_training_data="https://laion.ai/blog/laion-5b/",
    framework=["PyTorch"],
    reference="https://huggingface.co/QuanSun/EVA-CLIP",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=laion_2b,
    citation=EVA_CLIP_CITATION,
)


EVA02_CLIP_bigE_14_plus = ModelMeta(
    loader=evaclip_loader,
    name="QuanSun/EVA02-CLIP-bigE-14-plus",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="11afd202f2ae80869d6cef18b1ec775e79bd8d12",
    release_date="2023-04-26",
    modalities=["image", "text"],
    n_parameters=5_000_000_000,
    n_embedding_parameters=None,
    memory_usage_mb=19073,
    max_tokens=77,
    embed_dim=1024,
    license="mit",
    open_weights=True,
    public_training_code=training_code,
    public_training_data="https://laion.ai/blog/laion-5b/",
    framework=["PyTorch"],
    reference="https://huggingface.co/QuanSun/EVA-CLIP",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=laion_2b,
    citation=EVA_CLIP_CITATION,
)



from .audio_caps import AudioCapsA2TRetrieval, AudioCapsT2ARetrieval
from .belebele_retrieval import BelebeleRetrieval
from .common_voice import (
    CommonVoiceMini17A2TRetrieval,
    CommonVoiceMini17T2ARetrieval,
    CommonVoiceMini21A2TRetrieval,
    CommonVoiceMini21T2ARetrieval,
)
from .cross_lingual_semantic_discrimination_wmt19 import (
    CrossLingualSemanticDiscriminationWMT19,
)
from .cross_lingual_semantic_discrimination_wmt21 import (
    CrossLingualSemanticDiscriminationWMT21,
)
from .cur_ev1_retrieval import CUREv1Retrieval
from .euro_pirq_retrieval import EuroPIRQRetrieval
from .fleurs import FleursA2TRetrieval, FleursT2ARetrieval
from .google_svq import GoogleSVQA2TRetrieval, GoogleSVQT2ARetrieval
from .indic_qa_retrieval import IndicQARetrieval
from .jam_alt import (
    JamAltArtistA2ARetrieval,
    JamAltLyricA2TRetrieval,
    JamAltLyricT2ARetrieval,
)
from .jina_vdr_bench_retrieval import (
    JinaVDRAirbnbSyntheticRetrieval,
    JinaVDRArabicChartQARetrieval,
    JinaVDRArabicInfographicsVQARetrieval,
    JinaVDRArxivQARetrieval,
    JinaVDRAutomobileCatelogRetrieval,
    JinaVDRBeveragesCatalogueRetrieval,
    JinaVDRChartQARetrieval,
    JinaVDRCharXivOCRRetrieval,
    JinaVDRDocQAAI,
    JinaVDRDocQAEnergyRetrieval,
    JinaVDRDocQAGovReportRetrieval,
    JinaVDRDocQAHealthcareIndustryRetrieval,
    JinaVDRDocVQARetrieval,
    JinaVDRDonutVQAISynHMPRetrieval,
    JinaVDREuropeanaDeNewsRetrieval,
    JinaVDREuropeanaEsNewsRetrieval,
    JinaVDREuropeanaFrNewsRetrieval,
    JinaVDREuropeanaItScansRetrieval,
    JinaVDREuropeanaNlLegalRetrieval,
    JinaVDRGitHubReadmeRetrieval,
    JinaVDRHindiGovVQARetrieval,
    JinaVDRHungarianDocQARetrieval,
    JinaVDRInfovqaRetrieval,
    JinaVDRJDocQARetrieval,
    JinaVDRJina2024YearlyBookRetrieval,
    JinaVDRMedicalPrescriptionsRetrieval,
    JinaVDRMMTabRetrieval,
    JinaVDRMPMQARetrieval,
    JinaVDROpenAINewsRetrieval,
    JinaVDROWIDChartsRetrieval,
    JinaVDRPlotQARetrieval,
    JinaVDRRamensBenchmarkRetrieval,
    JinaVDRShanghaiMasterPlanRetrieval,
    JinaVDRShiftProjectRetrieval,
    JinaVDRStanfordSlideRetrieval,
    JinaVDRStudentEnrollmentSyntheticRetrieval,
    JinaVDRTabFQuadRetrieval,
    JinaVDRTableVQARetrieval,
    JinaVDRTatQARetrieval,
    JinaVDRTQARetrieval,
    JinaVDRTweetStockSyntheticsRetrieval,
    JinaVDRWikimediaCommonsDocumentsRetrieval,
    JinaVDRWikimediaCommonsMapsRetrieval,
)
from .mintaka_retrieval import MintakaRetrieval
from .miracl_retrieval import (
    MIRACLRetrieval,
    MIRACLRetrievalHardNegatives,
    MIRACLRetrievalHardNegativesV2,
)
from .miracl_vision_retrieval import MIRACLVisionRetrieval
from .mkqa_retrieval import MKQARetrieval
from .mlqa_retrieval import MLQARetrieval
from .mr_tidy_retrieval import MrTidyRetrieval
from .multi_long_doc_retrieval import MultiLongDocRetrieval
from .neu_clir2022_retrieval import (
    NeuCLIR2022Retrieval,
    NeuCLIR2022RetrievalHardNegatives,
)
from .neu_clir2023_retrieval import (
    NeuCLIR2023Retrieval,
    NeuCLIR2023RetrievalHardNegatives,
)
from .public_health_qa_retrieval import PublicHealthQARetrieval
from .ru_sci_bench_retrieval import RuSciBenchCiteRetrieval, RuSciBenchCociteRetrieval
from .statcan_dialogue_dataset_retrieval import StatcanDialogueDatasetRetrieval
from .vdr_multilingual_retrieval import VDRMultilingualRetrieval
from .vidore2_bench_retrieval import (
    Vidore2BioMedicalLecturesRetrieval,
    Vidore2EconomicsReportsRetrieval,
    Vidore2ESGReportsHLRetrieval,
    Vidore2ESGReportsRetrieval,
)
from .vidore3_bench_retrieval import (
    Vidore3ComputerScienceRetrieval,
    Vidore3ComputerScienceRetrievalv2,
    Vidore3EnergyRetrieval,
    Vidore3EnergyRetrievalv2,
    Vidore3FinanceEnRetrieval,
    Vidore3FinanceEnRetrievalv2,
    Vidore3FinanceFrRetrieval,
    Vidore3FinanceFrRetrievalv2,
    Vidore3HrRetrieval,
    Vidore3HrRetrievalv2,
    Vidore3IndustrialRetrieval,
    Vidore3IndustrialRetrievalv2,
    Vidore3NuclearRetrieval,
    Vidore3NuclearRetrievalv2,
    Vidore3PharmaceuticalsRetrieval,
    Vidore3PharmaceuticalsRetrievalv2,
    Vidore3PhysicsRetrieval,
    Vidore3PhysicsRetrievalv2,
    Vidore3TelecomRetrieval,
    Vidore3TelecomRetrievalv2,
)
from .web_faq_retrieval import WebFAQRetrieval
from .wikipedia_retrieval_multilingual import WikipediaRetrievalMultilingual
from .wit_t2i_retrieval import WITT2IRetrieval
from .x_flickr30k_co_t2i_retrieval import XFlickr30kCoT2IRetrieval
from .x_market_retrieval import XMarket
from .x_qu_ad_retrieval import XQuADRetrieval
from .xm3600_t2i_retrieval import XM3600T2IRetrieval
from .xpqa_retrieval import XPQARetrieval

__all__ = [
    "AudioCapsA2TRetrieval",
    "AudioCapsT2ARetrieval",
    "BelebeleRetrieval",
    "CUREv1Retrieval",
    "CommonVoiceMini17A2TRetrieval",
    "CommonVoiceMini17T2ARetrieval",
    "CommonVoiceMini21A2TRetrieval",
    "CommonVoiceMini21T2ARetrieval",
    "CrossLingualSemanticDiscriminationWMT19",
    "CrossLingualSemanticDiscriminationWMT21",
    "EuroPIRQRetrieval",
    "FleursA2TRetrieval",
    "FleursT2ARetrieval",
    "GoogleSVQA2TRetrieval",
    "GoogleSVQT2ARetrieval",
    "IndicQARetrieval",
    "JamAltArtistA2ARetrieval",
    "JamAltLyricA2TRetrieval",
    "JamAltLyricT2ARetrieval",
    "JinaVDRAirbnbSyntheticRetrieval",
    "JinaVDRArabicChartQARetrieval",
    "JinaVDRArabicInfographicsVQARetrieval",
    "JinaVDRArxivQARetrieval",
    "JinaVDRAutomobileCatelogRetrieval",
    "JinaVDRBeveragesCatalogueRetrieval",
    "JinaVDRCharXivOCRRetrieval",
    "JinaVDRChartQARetrieval",
    "JinaVDRDocQAAI",
    "JinaVDRDocQAEnergyRetrieval",
    "JinaVDRDocQAGovReportRetrieval",
    "JinaVDRDocQAHealthcareIndustryRetrieval",
    "JinaVDRDocVQARetrieval",
    "JinaVDRDonutVQAISynHMPRetrieval",
    "JinaVDREuropeanaDeNewsRetrieval",
    "JinaVDREuropeanaEsNewsRetrieval",
    "JinaVDREuropeanaFrNewsRetrieval",
    "JinaVDREuropeanaItScansRetrieval",
    "JinaVDREuropeanaNlLegalRetrieval",
    "JinaVDRGitHubReadmeRetrieval",
    "JinaVDRHindiGovVQARetrieval",
    "JinaVDRHungarianDocQARetrieval",
    "JinaVDRInfovqaRetrieval",
    "JinaVDRJDocQARetrieval",
    "JinaVDRJina2024YearlyBookRetrieval",
    "JinaVDRMMTabRetrieval",
    "JinaVDRMPMQARetrieval",
    "JinaVDRMedicalPrescriptionsRetrieval",
    "JinaVDROWIDChartsRetrieval",
    "JinaVDROpenAINewsRetrieval",
    "JinaVDRPlotQARetrieval",
    "JinaVDRRamensBenchmarkRetrieval",
    "JinaVDRShanghaiMasterPlanRetrieval",
    "JinaVDRShiftProjectRetrieval",
    "JinaVDRStanfordSlideRetrieval",
    "JinaVDRStudentEnrollmentSyntheticRetrieval",
    "JinaVDRTQARetrieval",
    "JinaVDRTabFQuadRetrieval",
    "JinaVDRTableVQARetrieval",
    "JinaVDRTatQARetrieval",
    "JinaVDRTweetStockSyntheticsRetrieval",
    "JinaVDRWikimediaCommonsDocumentsRetrieval",
    "JinaVDRWikimediaCommonsMapsRetrieval",
    "MIRACLRetrieval",
    "MIRACLRetrievalHardNegatives",
    "MIRACLRetrievalHardNegativesV2",
    "MIRACLVisionRetrieval",
    "MKQARetrieval",
    "MLQARetrieval",
    "MintakaRetrieval",
    "MrTidyRetrieval",
    "MultiLongDocRetrieval",
    "NeuCLIR2022Retrieval",
    "NeuCLIR2022RetrievalHardNegatives",
    "NeuCLIR2023Retrieval",
    "NeuCLIR2023RetrievalHardNegatives",
    "PublicHealthQARetrieval",
    "RuSciBenchCiteRetrieval",
    "RuSciBenchCociteRetrieval",
    "StatcanDialogueDatasetRetrieval",
    "VDRMultilingualRetrieval",
    "Vidore2BioMedicalLecturesRetrieval",
    "Vidore2ESGReportsHLRetrieval",
    "Vidore2ESGReportsRetrieval",
    "Vidore2EconomicsReportsRetrieval",
    "Vidore3ComputerScienceRetrieval",
    "Vidore3ComputerScienceRetrievalv2",
    "Vidore3EnergyRetrieval",
    "Vidore3EnergyRetrievalv2",
    "Vidore3FinanceEnRetrieval",
    "Vidore3FinanceEnRetrievalv2",
    "Vidore3FinanceFrRetrieval",
    "Vidore3FinanceFrRetrievalv2",
    "Vidore3HrRetrieval",
    "Vidore3HrRetrievalv2",
    "Vidore3IndustrialRetrieval",
    "Vidore3IndustrialRetrievalv2",
    "Vidore3NuclearRetrieval",
    "Vidore3NuclearRetrievalv2",
    "Vidore3PharmaceuticalsRetrieval",
    "Vidore3PharmaceuticalsRetrievalv2",
    "Vidore3PhysicsRetrieval",
    "Vidore3PhysicsRetrievalv2",
    "Vidore3TelecomRetrieval",
    "Vidore3TelecomRetrievalv2",
    "WITT2IRetrieval",
    "WebFAQRetrieval",
    "WikipediaRetrievalMultilingual",
    "XFlickr30kCoT2IRetrieval",
    "XM3600T2IRetrieval",
    "XMarket",
    "XPQARetrieval",
    "XQuADRetrieval",
]



from mteb.abstasks.sts import AbsTaskSTS
from mteb.abstasks.task_metadata import TaskMetadata


class UkrSedUASmallSTSv1(AbsTaskSTS):
    column_names = ("query", "passage")
    metadata = TaskMetadata(
        name="UkrSedUASmallSTSv1",
        description="Small (100k+) synthetic dataset for fine-tuning text embedding models for Ukrainian language (STS task)",
        reference="https://huggingface.co/datasets/suntez13/sed-ua-small-sts-v1",
        type="STS",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["ukr-Cyrl"],
        main_score="cosine_spearman",
        dataset={
            "path": "mteb/UkrSedUASmallSTSv1",
            "revision": "d83d1c605ce02b53f03112349d7966ab692590b1",
        },
        date=("2025-03-24", "2025-03-24"),
        domains=["Constructed"],
        task_subtypes=[],
        license="bsd-3-clause",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@proceedings{SED-UA-small2025,
  author = {Oleksandr Mediakov and Dmytro Martjanov and Vasyl Lytvyn},
  booktitle = {Proceedings of the Information Systems and Networks (SISN), Volume 17},
  doi = {10.23939/sisn2025.17.403},
  pages = {403--410},
  publisher = {Lviv Polytechnic National University},
  title = {SED-UA-Small: Ukrainian Synthetic Dataset for Text Embedding Models},
  url = {https://science.lpnu.ua/sisn/all-volumes-and-issues/volume-17-2025/sed-ua-small-ukrainian-synthetic-dataset-text-embedding},
  year = {2025},
}
""",
    )



from .core17_instruction_retrieval import Core17InstructionRetrieval
from .news21_instruction_retrieval import News21InstructionRetrieval
from .robust04_instruction_retrieval import Robust04InstructionRetrieval

__all__ = [
    "Core17InstructionRetrieval",
    "News21InstructionRetrieval",
    "Robust04InstructionRetrieval",
]
